/*************************************************************************
*
* BG Office Project
* Copyright (C) 2000-2004 Radostin Radnev <radnev@yahoo.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*************************************************************************/


/*
* $Id: main.cpp,v 1.3 2005/03/12 09:14:46 radnev Exp $
*/


#include <sys/types.h>
#include <time.h>

#ifdef WITH_KDE
#include <kapp.h>
#include <kcmdlineargs.h>
#include <kmessagebox.h>
#else
#include <qapplication.h>
#include <qmessagebox.h>
#endif

#include "config.h"
#include "properties.h"
#include "encodechar.h"
#include "win.h"
#include "splashscreen.h"


#ifdef DATA_DIR_CONFIG
static const char DATA_DIR[] = DATA_DIR_CONFIG;
#else
static const char DATA_DIR[] = "/usr/local/share/bgoffice/";
#endif


QTextCodec* Converter1251;

const char MISSING_DATA[] =
	"Cannot find dictionary files.\n"
	"Please look at README for additional information and help or "
	"visit http://bgoffice.sourceforge.net to download dictionaries.";


int main(int argc, char **argv) {

	Properties *prop = new Properties(getenv("HOME"), ".kbgoffice");
	char *path = prop->getString("DataDir", DATA_DIR);
	char *dataDir = new char[strlen(path) + 2];
	strcpy(dataDir, path);
	if ((dataDir[0] != '\0') && (dataDir[strlen(dataDir) - 1] != '/')) {
		strcat(dataDir, "/");
	}

	if ((argc > 1) && (strcmp(argv[1], "--dump-data-dir") == 0)) {
		printf("%s\n", dataDir);
		return 0;
	}

	Converter1251 = QTextCodec::codecForName("CP1251");
#ifdef WITH_KDE
	KCmdLineArgs::init(argc, argv, "kbgoffice", "kbgoffice", "BG Office Assistant for KDE", VERSION, false);
	KApplication a(true, true);
#else
	QApplication a(argc, argv);
#endif

	MySplashScreen *splash = NULL;
	time_t start;
	bool useSplashScreen = prop->getBool("UseSplashScreen", true);
	if (useSplashScreen) {
		QPixmap pixmap(QString(dataDir) + "kbgoffice_splash.png");
		splash = new MySplashScreen(pixmap);
		(void) time(&start);
	}

	// Translation file for application strings
	QTranslator qtr(0);
	qtr.load(QString(dataDir) + "kbgoffice_" + QTextCodec::locale(), ".");
	a.installTranslator(&qtr);

	// Main widged
	Win win(prop, dataDir);
	a.setMainWidget(&win);

	// Check data
	if (!win.isDataOk()) {
		if ((useSplashScreen) && (splash != NULL)) {
			// Close splash screen
			splash->close();
			delete splash;
		}

		// Center app, otherwise error win appears in center of app
		QRect scr = QApplication::desktop()->screenGeometry();
		win.move(scr.center() - win.rect().center());

#ifdef WITH_KDE
		KMessageBox::sorry(0, W2U(MISSING_DATA), win.caption());
#else
		QMessageBox::critical(0, win.caption(), W2U(MISSING_DATA));
#endif

		a.exit(1);
		return 1;
	}

	win.show();

	if ((useSplashScreen) && (splash != NULL)) {
		// Wait couple of seconds before to hide splash screen
		time_t end;
		do {
			(void) time(&end);
		} while ((int)(end - start) < 2);
		splash->finish(&win);
		delete splash;
	}

	return a.exec();
}
