/*************************************************************************
*
* systemtray.cpp
* Copyright (C) 2002 by Thanomsub Noppaburana <donga_n@yahoo.com>
*
* Contributed to BG Office project by Iliyan Peychev <peychevi@yahoo.com>
* Maintained by Radostin Radnev <radnev@yahoo.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*************************************************************************/


#ifdef WITH_KDE


#include <qcursor.h>
#include <qmessagebox.h>
#include <qtooltip.h>
#include <qtimer.h>
#include <qwhatsthis.h>

#include <klocale.h>
#include <kcursor.h>
#include <kiconloader.h>
#include <kpopupmenu.h>

#include <kwinmodule.h>

#include <kmainwindow.h>
#include <kdebug.h>
#include <kapp.h>

#include "systemtray.h"
#include "win.h"


/*
* This class is used to dock window to system tray.
* It works only if compiled with KDE.
*
* $Id: systemtray.cpp,v 1.3 2004/11/10 21:20:22 radnev Exp $
*/


CSystemTray::CSystemTray(QWidget *parent, const char *name, QPixmap qpxmIcon): KSystemTray(parent, name) {
	trayMenu = new KPopupMenu(this);
	trayMenu->insertTitle(qpxmIcon, tr("bgOffice Assistant"));
	globalKeys = new KGlobalAccel(this);
	globalKeys->updateConnections();
	watchItem = trayMenu->insertItem(SmallIcon("klipper", KIcon::SizeSmall), tr("&Watch Clipboard"), this, SLOT(slotToggleWatchClipboard()));
	trayMenu->insertSeparator();
	restoreItem = trayMenu->insertItem(tr("&Restore"), this, SLOT(switchRestore()));
	trayMenu->insertItem(SmallIcon("configure", KIcon::SizeSmall), tr("&Configure..."), this, SLOT(slotConfig()));
	trayMenu->insertSeparator();
	trayMenu->insertItem(SmallIcon("exit", KIcon::SizeSmall), tr("&Quit"), this, SLOT(slotQuit()));
	connect(trayMenu, SIGNAL(aboutToShow()), this, SLOT(prepareMenu()));
	setBackgroundMode(X11ParentRelative);
	QToolTip::add(this, tr("bgOffice Assistant"));
}


CSystemTray::~CSystemTray() {
	if (trayMenu != NULL) {
		delete trayMenu;
	}
}


void CSystemTray::prepareMenu() {
	if (parentWidget()->isVisible()) {
		trayMenu->changeItem(restoreItem, tr("Mi&nimize"));
	} else {
		trayMenu->changeItem(restoreItem, tr("&Restore"));
	}
	trayMenu->setItemChecked(watchItem, ((Win*)parentWidget())->isWatchClipboard());
}


KPopupMenu *CSystemTray::menu() {
	return trayMenu;
}


void CSystemTray::mousePressEvent(QMouseEvent *ev) {
	if (ev->button() == RightButton) {
		trayMenu->exec(QCursor::pos());
	} else {
		KSystemTray::mousePressEvent(ev);
	}
}


void CSystemTray::mouseReleaseEvent(QMouseEvent *ev) {
	KSystemTray::mouseReleaseEvent(ev);
}


void CSystemTray::switchRestore() {
	if (parentWidget()->isVisible()) {
		parentWidget()->hide();
	} else {
		parentWidget()->show();
	}
}


void CSystemTray::slotConfig() {
	((Win*) parentWidget())->showOptions();
}


void CSystemTray::slotToggleWatchClipboard() {
	bool bAdvWatch = FALSE;
	bool bWatch = !((Win*)parentWidget())->isWatchClipboard();
	if (bWatch == FALSE) {
		bAdvWatch = FALSE;
	}
	((Win*)parentWidget())->setWatchClipboard(bWatch, ((Win*) parentWidget())->isAdvWatchClipboard());
	trayMenu->setItemChecked(watchItem, ((Win*)parentWidget())->isWatchClipboard());
}


void CSystemTray::slotQuit() {
	kapp->quit();
}

// Include this stuff, because moc doesn't understand conditional compilation
#include "systemtray.moc.cpp"

#endif
