/*************************************************************************
*
* BG Office Project
* Copyright (C) 2000-2004 Radostin Radnev <radnev@yahoo.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <qwidget.h>
#include <qdialog.h>
#include <qfont.h>
#include <qkeycode.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qtextedit.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qstring.h>
#include <qframe.h>
#include <qlayout.h>
#include <qwhatsthis.h>
#include <qcolor.h>

#include "encodechar.h"
#include "test.h"


/*
* This is main test/exam window
* Written by Radostin Radnev - radnev@yahoo.com
*
* $Id: test.cpp,v 1.2 2004/10/26 21:05:57 radnev Exp $
*/


// Some constants
static const int  MAX_ANSWER_LEN    = 100;
static const int  MIN_LEFT_WIDTH    = 100;
static const int  SPACE_IN_DIALOGS  = 10;

// Welcome, help and other messages
static const char CAPTION[]         = QT_TR_NOOP("bgOffice Test Suite");
static const char FOUND[]           = QT_TR_NOOP(":-)");
static const char NOT_FOUND[]       = QT_TR_NOOP(":-(");
static const char END_OF_TEST[]     = QT_TR_NOOP("End of session. Start again or exit.");
static const char WELCOME_TEXT[]    = QT_TR_NOOP("To begin new test press button \"New Test\".");
static const char HELP_TEXT[]       = QT_TR_NOOP(
	"Enter answer that you think is correct.<br>"
	"Use <b>comma</b> (<b>,</b>) to separate two or more answers that you may have.<br>");

// Status Info and word format message
static const char STATUS_INFO[] = QT_TR_NOOP(" Correct  %1%  ( %2 / %3 )     Asked  %4%  ( %5 / %6 )");
static const char ASKED[]       = QT_TR_NOOP("%1.  %2");

// Color objects for found and not found
static const QColor COLOR_FOUND(0, 255, 0);
static const QColor COLOR_NOT_FOUND(255, 0, 0);

// Default options
static const char DEFAULT_GEOMETRY[]              = "50, 50, 350, 250";
static const int  DEFAULT_TRANSLATE               = 1;
static const int  DEFAULT_SHOW_ANSWER             = 1;
static const int  DEFAULT_SESSION                 = 0;
static const int  DEFAULT_QUESTIONS               = 100;
static const int  DEFAULT_LEVEL                   = 3;

// Min and max allowed numbers
static const int  MIN_SHOW_ANSWER                 = 1;
static const int  MAX_SHOW_ANSWER                 = 5;
static const int  MIN_SESSION                     = 0;
static const int  MAX_SESSION                     = 1000000;
static const int  MIN_QUESTIONS                   = 10;
static const int  MAX_QUESTIONS                   = 10000;
static const int  MIN_LEVEL                       = 1;
static const int  MAX_LEVEL                       = 5;

// Properties keys
static const char OPTION_GEOMETRY[]               = "Test.Geometry";
static const char OPTION_TRANSLATE[]              = "Test.Translate";
static const char OPTION_SHOW_ANSWER[]            = "Test.ShowAnswer";
static const char OPTION_SESSION[]                = "Test.Session";
static const char OPTION_QUESTIONS[]              = "Test.Questions";
static const char OPTION_LEVEL[]                  = "Test.Level";


Test::Test(QWidget *parent, const char *name, TranslatorManager *pDictionary, Properties *pProp, const QFont *pFont, const int pBorderWidth):QDialog(parent, name, true) {

	setCaption(tr(CAPTION));
	trManager = pDictionary;
	dictionary = NULL;
	borderWidth = pBorderWidth;
	prop = pProp;
	font = QFont(*pFont);
	setFont(font);

	int X = 0, Y = 0, W = 0, H = 0;
	sscanf(DEFAULT_GEOMETRY, "%d, %d, %d, %d", &X, &Y, &W, &H);
	sscanf(prop->getString(OPTION_GEOMETRY, DEFAULT_GEOMETRY), "%d, %d, %d, %d", &X, &Y, &W, &H);
	resize(QSize(W, H));
	move(QPoint(X, Y));

	translate = prop->getInt(OPTION_TRANSLATE, DEFAULT_TRANSLATE);
	showAnswer = prop->getInt(OPTION_SHOW_ANSWER, DEFAULT_SHOW_ANSWER, MIN_SHOW_ANSWER, MAX_SHOW_ANSWER);
	session = prop->getInt(OPTION_SESSION, DEFAULT_SESSION, MIN_SESSION, MAX_SESSION);
	questions = prop->getInt(OPTION_QUESTIONS, DEFAULT_QUESTIONS, MIN_QUESTIONS, MAX_QUESTIONS);
	level = prop->getInt(OPTION_LEVEL, DEFAULT_LEVEL, MIN_LEVEL, MAX_LEVEL);
	current = questions + 1;
	correct = 0;

	// Create and set Answer (text field)
	answer = new QLineEdit(W2U(""), this);
	CHECK_PTR(answer);
	answer->setMaxLength(MAX_ANSWER_LEN);
	connect(answer, SIGNAL(returnPressed()), this, SLOT(answer_EnterPressed()));
	answer->adjustSize();

	// Create and set label question
	question = new QLabel(this);
	CHECK_PTR(question);
	question->setAlignment(QLabel::AlignLeft | QLabel::AlignVCenter);
	question->setMaximumHeight(answer->height());

	// Create and set label word
	found = new QLabel(this);
	CHECK_PTR(found);
	found->setAlignment(QLabel::AlignCenter | QLabel::AlignVCenter);
	found->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	found->setMaximumWidth(answer->height());
	found->setMinimumWidth(answer->height());
	found->setMaximumHeight(answer->height());
	found->setMinimumHeight(answer->height());

	// Create and set label word
	word = new QLabel(this);
	CHECK_PTR(word);
	word->setAlignment(QLabel::AlignLeft | QLabel::AlignVCenter);
	word->setMaximumHeight(answer->height());

	// Create and set button new test
	bnew = new QPushButton(tr("&New Test"), this);
	CHECK_PTR(bnew);
	connect(bnew, SIGNAL(clicked()), this, SLOT(button_New()));
	bnew->setFocusPolicy(QPushButton::NoFocus);
	bnew->setAutoDefault(false);

	// Create and set button close
	bclose = new QPushButton(tr("&Close"), this);
	CHECK_PTR(bclose);
	connect(bclose, SIGNAL(clicked()), this, SLOT(reject()));
	bclose->setFocusPolicy(QPushButton::NoFocus);
	bclose->setAutoDefault(false);

	// Create and set button help
	bhelp = new QPushButton(tr("&Help"), this);
	CHECK_PTR(bhelp);
	connect(bhelp, SIGNAL(clicked()), this, SLOT(button_Help()));
	bhelp->setFocusPolicy(QPushButton::NoFocus);
	bhelp->setAutoDefault(false);

	// Create and set result to show
	result = new QTextEdit(this);
	result->setReadOnly(true);
	CHECK_PTR(result);

	// Create and set label info
	info = new QLabel(this);
	CHECK_PTR(info);
	info->setAlignment(QLabel::AlignLeft | QLabel::AlignVCenter);
	info->setFrameStyle(QFrame::Panel | QFrame::Sunken);

	QVBoxLayout *mainLayout = new QVBoxLayout(this, borderWidth);
	QVBoxLayout *labelLayout = new QVBoxLayout(borderWidth);
	QVBoxLayout *buttonLayout = new QVBoxLayout(borderWidth);
	QHBoxLayout *upperLayout = new QHBoxLayout(borderWidth);
	QHBoxLayout *statusLayout = new QHBoxLayout(borderWidth);

	statusLayout->addWidget(found);
	statusLayout->addSpacing(borderWidth);
	statusLayout->addWidget(word);

	labelLayout->addWidget(question);
	labelLayout->addSpacing(borderWidth);
	labelLayout->addWidget(answer);
	labelLayout->addSpacing(borderWidth);
	labelLayout->addLayout(statusLayout);

	buttonLayout->addWidget(bnew);
	buttonLayout->addWidget(bclose);
	buttonLayout->addWidget(bhelp);

	upperLayout->addLayout(labelLayout);
	upperLayout->addLayout(buttonLayout);

	mainLayout->addLayout(upperLayout);
	mainLayout->addWidget(result);
	mainLayout->addWidget(info);

	// Set focus
	answer->setFocus();
	question->setText(tr(WELCOME_TEXT));
}


Test::~Test() {
	prop->setInt(OPTION_TRANSLATE, translate);
	prop->setInt(OPTION_SHOW_ANSWER, showAnswer);
	prop->setInt(OPTION_SESSION, session);
	prop->setInt(OPTION_QUESTIONS, questions);
	prop->setInt(OPTION_LEVEL, level);
	if (!isMinimized()) {
		char val[64];
		sprintf(val, "%d, %d, %d, %d", pos().x(), pos().y(), size().width(), size().height());
		prop->setString(OPTION_GEOMETRY, val);
	}
	delete answer;
	delete bnew;
	delete bclose;
	delete bhelp;
	delete question;
	delete found;
	delete word;
	delete info;
	delete result;
	if (dictionary != NULL) {
		delete dictionary;
	}
}


void Test::answer_EnterPressed() {
	if (current > questions) {
		// Do nothing
		answer->clear();
		return;
	}
	// Clear fields
	result->clear();
	// Check for new sessoin
	if (current == 0) {
		// Start new session
		if (session == 0) {
			srand(time(NULL));
		} else {
			srand(session);
		}
		answer->clear();
		word->clear();
		found->clear();
		found->setBackgroundColor(word->backgroundColor());
	} else {
		// Check result
		if (dictionary->testWord(U2W(answer->text()))) {
			correct++;
			found->setBackgroundColor(COLOR_FOUND);
			found->setText(tr(FOUND));
		} else {
			found->setBackgroundColor(COLOR_NOT_FOUND);
			found->setText(tr(NOT_FOUND));
			if (((showAnswer == 2) && ((answer->text()).length() > 0)) || ((showAnswer == 3) && ((answer->text()).length() == 0)) || (showAnswer == 4)) {
				result->setText(W2U(dictionary->getResult()));
				result->sync();
				result->setCursorPosition(0, 0);
			}
		}
		word->setText(answer->text());
		answer->clear();
		if (showAnswer == 1) {
			result->setText(W2U(dictionary->getResult()));
			result->sync();
			result->setCursorPosition(0, 0);
		}
	}
	// Show status
	info->setText(tr(STATUS_INFO)
			.arg((current == 0 ? 0 : (correct*100)/current))
			.arg(correct)
			.arg(current)
			.arg((questions == 0 ? 0 : (current*100)/questions))
			.arg(current)
			.arg(questions));
	// Increase current ponter
	current++;
	// Generata next random word or end of test
	if (current > questions) {
		question->setText(tr(END_OF_TEST));
	} else {
		question->setText(tr(ASKED)
				.arg(current)
				.arg(W2U(dictionary->getNextRandomWord())));
	}
}


void Test::button_New() {

	QDialog options(this, "NewTest", true);
	options.setCaption(tr("New Test"));

	QVBoxLayout optionsLayout(&options, SPACE_IN_DIALOGS);
	QGridLayout gridLayout(SPACE_IN_DIALOGS, 2, 7);
	QHBoxLayout buttonLayout(SPACE_IN_DIALOGS);

	QLabel ltf(tr("Translate:"), &options);
	ltf.setAlignment(QLabel::AlignRight | QLabel::AlignVCenter);

	QComboBox tf(false, &options);
	for(int i = 0; i < trManager->sizeOfTestDictionaries(); i++) {
		tf.insertItem(W2U(trManager->getTestDictionary(i).name.c_str()));
	}
	if (translate - 1 < trManager->sizeOfTestDictionaries()) {
		tf.setCurrentItem(translate - 1);
	} else {
		tf.setCurrentItem(0);
	}

	QLabel lsa(tr("Show answer:"), &options);
	lsa.setAlignment(QLabel::AlignRight | QLabel::AlignVCenter);

	QComboBox sa(false, &options);
	sa.insertItem(tr("Every time"));
	sa.insertItem(tr("In wrong answer"));
	sa.insertItem(tr("In empty answer"));
	sa.insertItem(tr("In wrong or empty answer"));
	sa.insertItem(tr("Never"));
	sa.setCurrentItem(showAnswer - 1);

	QLabel lsn(tr("Session (0 - random):"), &options);
	lsn.setAlignment(QLabel::AlignRight | QLabel::AlignVCenter);

	QSpinBox sn(MIN_SESSION, MAX_SESSION, 1, &options);
	sn.setValue(session);

	QLabel lqn(tr("Number of questions:"), &options);
	lqn.setAlignment(QLabel::AlignRight | QLabel::AlignVCenter);

	QSpinBox qn(MIN_QUESTIONS, MAX_QUESTIONS, 10, &options);
	qn.setValue(questions);

	QLabel llv(tr("Test difficult level:"), &options);
	llv.setAlignment(QLabel::AlignRight | QLabel::AlignVCenter);

	QComboBox lv(false, &options);
	lv.insertItem(tr("Novice"));
	lv.insertItem(tr("Beginner"));
	lv.insertItem(tr("Intermediate"));
	lv.insertItem(tr("Specialist"));
	lv.insertItem(tr("Expert"));
	lv.setCurrentItem(level - 1);

	// Create buttons
	QPushButton ok(tr("&OK"), &options);
	ok.setDefault(true);
	connect(&ok, SIGNAL(clicked()), &options, SLOT(accept()));

	QPushButton cancel(tr("&Cancel"), &options);
	connect(&cancel, SIGNAL(clicked()), &options, SLOT(reject()));
	cancel.adjustSize();

	gridLayout.addWidget(&ltf, 0, 0);
	gridLayout.addWidget(&tf, 0, 1);
	gridLayout.addWidget(&lsa, 1, 0);
	gridLayout.addWidget(&sa, 1, 1);
	gridLayout.addRowSpacing(2, SPACE_IN_DIALOGS*2);

	gridLayout.addWidget(&lsn, 3, 0);
	gridLayout.addWidget(&lqn, 4, 0);

	QHBoxLayout niceLayout(SPACE_IN_DIALOGS);
	QVBoxLayout addLayout(SPACE_IN_DIALOGS);
	addLayout.addWidget(&sn);
	addLayout.addWidget(&qn);
	niceLayout.addLayout(&addLayout);
	niceLayout.addStretch();

	gridLayout.addMultiCellLayout(&niceLayout, 3, 4, 1, 1);

	gridLayout.addRowSpacing(5, SPACE_IN_DIALOGS*2);

	QHBoxLayout levelLayout(SPACE_IN_DIALOGS);
	levelLayout.addWidget(&lv);
	levelLayout.addStretch();

	gridLayout.addWidget(&llv, 6, 0);
	gridLayout.addLayout(&levelLayout, 6, 1);

	buttonLayout.addStretch();
	buttonLayout.addWidget(&ok);
	buttonLayout.addWidget(&cancel);

	optionsLayout.addLayout(&gridLayout);
	optionsLayout.addSpacing(SPACE_IN_DIALOGS*3);
	optionsLayout.addLayout(&buttonLayout);

	// Execute dialog and check for OK button
	if (options.exec() == QDialog::Accepted) {
		translate = tf.currentItem() + 1;
		showAnswer = sa.currentItem() + 1;
		level = lv.currentItem() + 1;
		session = sn.value();
		questions = qn.value();
		current = 0;
		correct = 0;
		if (dictionary != NULL) {
			delete dictionary;
		}
		dictionary = trManager->getTestDictionaryObject(translate - 1, level);
		answer->setFocus();
		answer_EnterPressed();
	}

}


void Test::button_Help() {
	result->setText(tr(HELP_TEXT));
	result->sync();
	result->setCursorPosition(0, 0);
}
