/*************************************************************************
*
* watchclipboard.cpp
* Copyright (C) 2002 by Thanomsub Noppaburana <donga_n@yahoo.com>
*
* Contributed to BG Office project by Iliyan Peychev <peychevi@yahoo.com>
* Maintained by Radostin Radnev <radnev@yahoo.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*************************************************************************/


#ifdef WITH_KDE


#include <qtimer.h>
#include <kapplication.h>
#include <kmessagebox.h>
#include "watchclipboard.h"


/*
* This class is used to watch clipboard via Klipper.
* It works only if compiled with KDE.
*
* $Id: watchclipboard.cpp,v 1.2 2004/10/26 21:25:02 radnev Exp $
*/


CWatchClipboard::CWatchClipboard(): QObject(), watchTimer(NULL), _interval(1000), lastContent("") {
}


CWatchClipboard::~CWatchClipboard() {
	stopWatch();
	if (dcopClient != NULL) {
		dcopClient->detach();
	}
}


bool CWatchClipboard::becomeDCOPClient() {
	bool bSuccess = FALSE;
	dcopClient = KApplication::dcopClient();
	if (dcopClient == NULL) {
		return FALSE;
	}
	bSuccess = dcopClient->attach();
	if (!bSuccess) {
		return FALSE;
	}
	return TRUE;
}


void CWatchClipboard::setInterval(int iMSec) {
	if ( _interval != iMSec) {
		_interval = iMSec;
		if (watchTimer != NULL) {
			watchTimer->changeInterval(_interval);
		}
	}
}


int CWatchClipboard::getInterval() {
	return _interval;
}


void CWatchClipboard::startWatch() {
	stopWatch();
	watchTimer = new QTimer(this, "watchTimer");
	connect(watchTimer, SIGNAL(timeout()), this, SLOT(slotOnTimer()));
	watchTimer->start(_interval, false);
	bTimerEmabled = TRUE;
}


void CWatchClipboard::stopWatch() {
	if (watchTimer != NULL) {
		watchTimer->stop();
		delete watchTimer;
		watchTimer = NULL;
		bTimerEmabled = FALSE;
	}
}


bool CWatchClipboard::getTimerStatus() {
	return bTimerEmabled;
}


bool CWatchClipboard::invokeKlipper() {
	bool bSuccess = FALSE;
	QByteArray data, replyData;
	QDataStream arg( data, IO_WriteOnly );
	arg << 0;

	bSuccess = dcopClient->callAsync(
		"klipper",
		"klipper",
		"getClipboardHistoryItem(int)",
		data,
		this,
		SLOT(callBackClipboardData(int, const QCString&, const QByteArray&)));

	if (!bSuccess) {
		stopWatch();
		KMessageBox::sorry(0, tr("Error receiving data from Klipper.\nClipboard watch will be disabled."));
		return FALSE;
	}
	return TRUE;
}


void CWatchClipboard::callBackClipboardData(int callId, const QCString &replyType, const QByteArray &replyData) {
	QString result;
	QDataStream reply(replyData, IO_ReadOnly);
	if (replyType == "QString") {
		reply >> result;
	} else {
		stopWatch();
		KMessageBox::sorry(0, tr("Error receiving data from Klipper.\nClipboard watch will be disabled."));
	}
	if (lastContent != result) {
		lastContent = result;
		emit(clipboardChanged(result));
	}
}


void CWatchClipboard::slotOnTimer() {
	invokeKlipper();
}


// Include this stuff, because moc doesn't understand conditional compilation
#include <watchclipboard.moc.cpp>

#endif
