/*************************************************************************
*
* BG Office Project
* Copyright (C) 2000-2004 Radostin Radnev <radnev@yahoo.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <qwidget.h>
#include <qmainwindow.h>
#include <qdialog.h>
#include <qfont.h>
#include <qclipboard.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qtextbrowser.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qstring.h>
#include <qstylefactory.h>
#include <qtabdialog.h>
#include <qlayout.h>
#include <qwhatsthis.h>
#include <qcolor.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qbuttongroup.h>
#include <qhttp.h>
#include <qfile.h>
#include <qurl.h>
#include <qprocess.h>

#ifdef WITH_KDE
#include <kapp.h>
#include <kmenubar.h>
#include <kfontdialog.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#else
#include <qapplication.h>
#include <qfontdialog.h>
#include <qmenubar.h>
#endif

#include "config.h"

#include "encodechar.h"

#include "win.h"
#include "test.h"

// Include Icon and Image of office assistant
#include "../icons/kbgoffice_icon.xpm"
#include "../icons/kbgoffice_icon_mini.xpm"


/*
* This is main window class - GUI
* Written by Radostin Radnev - radnev@yahoo.com
*
* $Id: win.cpp,v 1.14 2005/11/25 22:26:35 radnev Exp $
*/



// Some Constants
static const int  MAX_WORD_LEN           = 70;
static const int  MAX_WORDS_ALLOWED      = 7;
static const int  SPACE_IN_DIALOGS       = 10;
static const char CAPTION[]              = QT_TR_NOOP("bgOffice Assistant");
static const char FOUND[]                = QT_TR_NOOP(":-)");
static const char NOT_FOUND[]            = QT_TR_NOOP(":-(");
static const char SIMILAR[]              = QT_TR_NOOP(":-|");
static const char MISSING_ABRR_FILE[]    = QT_TR_NOOP("<FONT color=\"red\">Error: Abbreviations file is missing.</FONT>");

static const QColor COLOR_FOUND(0, 255, 0);
static const QColor COLOR_NOT_FOUND(255, 0, 0);
static const QColor COLOR_SIMILAR(255, 255, 0);

static const char ABOUT_MESSAGE[] = QT_TR_NOOP(
	"bgOffice Assistant version %1\n"
	"Copyright (C) 2000-2005, Radostin Radnev\n"
	"\n"
	"Home page: http://bgoffice.sourceforge.net\n"
	"\n"
	"bgOffice Assistant comes with ABSOLUTELY\n"
	"NO WARRANTY. This is free software, and\n"
	"you are welcome to redistribute it under\n"
	"certain conditions. For details read file\n"
	"COPYING that comes with this package.");

static const char SEARCH_DICTIONARY[] = "http://www.infoplease.com/search.php3?in=all&query=";
static const char SEARCH_ENCYCLOPEDIA[] = "http://www.infoplease.com/search.php3?in=encyclopedia&query=";
static const char SEARCH_ALL[] = "http://www.infoplease.com/search.php3?in=encyclopedia&query=";

const char CREATION_TEMP_FILE_FAILED[] = QT_TR_NOOP(
	"Failed to create a temporaty file.\n"
	"Check your write permissions in \"/tmp\" directory.");

// Default options
static const int  DEFAULT_INPUT_FIELD_WIDTH        = 200;
static const int  DEFAULT_LEFT_PANEL_WIDTH         = 200;
static const int  DEFAULT_WORDS_IN_LIST            = 250;
static const char DEFAULT_WORDS_IN_HISTORY         = 50;
static const int  DEFAULT_BORDER_WIDTH             = 5;
static const bool DEFAULT_SHOW_TOOL_BAR            = false;
static const bool DEFAULT_DOCK_TOOL_BAR            = false;
static const bool DEFAULT_TRANSLATE_ON_EVERY_KEY   = true;
static const bool DEFAULT_WATCH_CLIPBOARD          = true;
static const bool DEFAULT_ADVANCED_CLIPBOARD       = false;
static const bool DEFAULT_ADVANCED_OPTIONS         = false;
static const bool DEFAULT_SEPARATE_MEANINGS        = true;
static const bool DEFAULT_TRY_WITHOUT_SUFFIX       = true;
static const bool DEFAULT_LATIN_OUTPUT             = false;
static const bool DEFAULT_LATIN_INPUT              = false;
static const char DEFAULT_FONT_NAME[]              = "*";
static const int  DEFAULT_FONT_SIZE                = 14;
static const int  DEFAULT_FONT_WEIGHT              = 50;
static const bool DEFAULT_FONT_ITALIC              = false;
static const char DEFAULT_GEOMETRY[]               = "50, 50, 500, 300";
static const char DEFAULT_SYSTEM_STYLE_NAME[]      = "Windows";
static const bool DEFAULT_ADV_SRCH_HIGHLIGHT       = true;
static const bool DEFAULT_ADV_SRCH_WHOLE_WORD      = true;
static const bool DEFAULT_ADV_SRCH_EXACT_PHRASE    = false;
static const bool DEFAULT_ADV_SRCH_ASD_IGNORE_KEY  = true;
static const bool DEFAULT_ADV_SRCH_ASD_IGNORE_HIS  = true;
static const bool DEFAULT_ADV_SRCH_ASD_ENTER       = true;
static const bool DEFAULT_ADV_SRCH_ASD_DO_NOT_REPL = true;
static const char DEFAULT_BROWSER_NAME[]           = "mozilla";
static const bool DEFAULT_USE_EXTERNAL_BROWSER     = true;
static const bool DEFAULT_USE_SPLASH_SCREEN        = true;


// Min & Max options
static const int  MAX_INPUT_FIELD_WIDTH = 500;
static const int  MIN_INPUT_FIELD_WIDTH = 25;
static const int  MAX_LEFT_PANEL_WIDTH  = 500;
static const int  MIN_LEFT_PANEL_WIDTH  = 0;
static const int  MAX_WORDS_IN_LIST     = 500;
static const int  MIN_WORDS_IN_LIST     = 0;
static const int  MAX_WORDS_IN_HISTORY  = 200;
static const int  MIN_WORDS_IN_HISTORY  = 0;
static const int  MAX_BORDER_WIDTH      = 25;
static const int  MIN_BORDER_WIDTH      = 0;


// Strings in registry
static const char OPTION_INPUT_FIELD_WIDTH[]        = "InputFieldWidth";
static const char OPTION_LEFT_PANEL_WIDTH[]         = "LeftPanelWidth";
static const char OPTION_WORDS_IN_LIST[]            = "WordsInList";
static const char OPTION_WORDS_IN_HISTORY[]         = "WordsInHistory";
static const char OPTION_BORDER_WIDTH[]             = "BorderWidth";
static const char OPTION_SHOW_TOOL_BAR[]            = "ShowToolBar";
static const char OPTION_DOCK_TOOL_BAR[]            = "DockToolBar";
static const char OPTION_SYSTEM_STYLE_NAME[]        = "SystemStyleName";
static const char OPTION_TRANSLATE_ON_EVERY_KEY[]   = "TraslateOnEveryKey";
static const char OPTION_WATCH_CLIPBOARD[]          = "WatchClipboard";
static const char OPTION_ADVANCED_CLIPBOARD[]       = "AdvancedClipboard";
static const char OPTION_ADVANCED_OPTIONS[]         = "AdvancedOptions";
static const char OPTION_SEPARATE_MEANINGS[]        = "SeparateMeanings";
static const char OPTION_TRY_WITHOUT_SUFFIX[]       = "TryWithoutSuffix";
static const char OPTION_FONT_NAME[]                = "FontName";
static const char OPTION_FONT_SIZE[]                = "FontSize";
static const char OPTION_FONT_WEIGHT[]              = "FontWeight";
static const char OPTION_FONT_ITALIC[]              = "FontItalic";
static const char OPTION_GEOMETRY[]                 = "Geometry";
static const char OPTION_LAST_USED_DICTIONARY[]     = "LastUsedDictionary";
static const char OPTION_ADV_SRCH_HIGHLIGHT[]       = "AdvSrch.Highlight";
static const char OPTION_ADV_SRCH_WHOLE_WORD[]      = "AdvSrch.WholeWord";
static const char OPTION_ADV_SRCH_EXACT_PHRASE[]    = "AdvSrch.EaxctPhrase";
static const char OPTION_ADV_SRCH_ASD_IGNORE_KEY[]  = "AdvSrch.IgnoreOnEveryKey";
static const char OPTION_ADV_SRCH_ASD_IGNORE_HIS[]  = "AdvSrch.IgnoreFromHistory";
static const char OPTION_ADV_SRCH_ASD_ENTER[]       = "AdvSrch.EnterSearch";
static const char OPTION_ADV_SRCH_ASD_DO_NOT_REPL[] = "AdvSrch.DoNotReplacePhrase";
static const char OPTION_DEFAULT_BROWSER[]          = "DefaultBrowser";
static const char OPTION_USE_EXTERNAL_BROWSER[]     = "UseExternalBrowser";
static const char OPTION_USE_SPLASH_SCREEN[]        = "UseSplashScreen";

Win::Win(Properties *pProp, char *pDataDir, QWidget *parent, const char *name):QMainWindow(parent, name) {

	// Keep reference to some vars
	prop = pProp;
	dataDir = pDataDir;

	// Create and Init Dictionary object
	dictionary = new TranslatorManager(dataDir);
	dataOk = dictionary->init();

	// Init some variables
	fromSystemHighlighted = false;
	fromListHighlighted = false;
	fromHistoryBuffer = false;
	fromListSelected = false;
	fromClipboard = false;
	fromSystem = false;
	fromTextChanged = false;
	fromAdvancedSearch = false;

	// Read data from ini file and set it
	inputFieldWidth = prop->getInt(OPTION_INPUT_FIELD_WIDTH, DEFAULT_INPUT_FIELD_WIDTH, MIN_INPUT_FIELD_WIDTH, MAX_INPUT_FIELD_WIDTH);
	leftPanelWidth = prop->getInt(OPTION_LEFT_PANEL_WIDTH, DEFAULT_LEFT_PANEL_WIDTH, MIN_LEFT_PANEL_WIDTH, MAX_LEFT_PANEL_WIDTH);
	wordsInList = prop->getInt(OPTION_WORDS_IN_LIST, DEFAULT_WORDS_IN_LIST, MIN_WORDS_IN_LIST, MAX_WORDS_IN_LIST);
	borderWidth = prop->getInt(OPTION_BORDER_WIDTH, DEFAULT_BORDER_WIDTH, MIN_BORDER_WIDTH, MAX_BORDER_WIDTH);
	wordsInHistory = prop->getInt(OPTION_WORDS_IN_HISTORY, DEFAULT_WORDS_IN_HISTORY, MIN_WORDS_IN_HISTORY, MAX_WORDS_IN_HISTORY);
	translateOnEveryKey = prop->getBool(OPTION_TRANSLATE_ON_EVERY_KEY, DEFAULT_TRANSLATE_ON_EVERY_KEY);
	watchClipboard = prop->getBool(OPTION_WATCH_CLIPBOARD, DEFAULT_WATCH_CLIPBOARD);
	advancedClipboard = prop->getBool(OPTION_ADVANCED_CLIPBOARD, DEFAULT_ADVANCED_CLIPBOARD);
	advancedOptions = prop->getBool(OPTION_ADVANCED_OPTIONS, DEFAULT_ADVANCED_OPTIONS);
	showToolBar = prop->getBool(OPTION_SHOW_TOOL_BAR, DEFAULT_SHOW_TOOL_BAR);
	dockToolBar = prop->getBool(OPTION_DOCK_TOOL_BAR, DEFAULT_DOCK_TOOL_BAR);
	advSrchIgnoreKey = prop->getBool(OPTION_ADV_SRCH_ASD_IGNORE_KEY, DEFAULT_ADV_SRCH_ASD_IGNORE_KEY);
	advSrchIgnoreHistory = prop->getBool(OPTION_ADV_SRCH_ASD_IGNORE_HIS, DEFAULT_ADV_SRCH_ASD_IGNORE_HIS);
	advSrchEnterSearch = prop->getBool(OPTION_ADV_SRCH_ASD_ENTER, DEFAULT_ADV_SRCH_ASD_ENTER);
	advSrchDoNotReplacePhrase = prop->getBool(OPTION_ADV_SRCH_ASD_DO_NOT_REPL, DEFAULT_ADV_SRCH_ASD_DO_NOT_REPL);
	browserName = prop->getString(OPTION_DEFAULT_BROWSER, DEFAULT_BROWSER_NAME);
	useExternalBrowser = prop->getBool(OPTION_USE_EXTERNAL_BROWSER, DEFAULT_USE_EXTERNAL_BROWSER);
	useSplashScreen = prop->getBool(OPTION_USE_SPLASH_SCREEN, DEFAULT_USE_SPLASH_SCREEN);

	if (dataOk) {
		dictionary->setSeparateMeanings(prop->getBool(OPTION_SEPARATE_MEANINGS, DEFAULT_SEPARATE_MEANINGS));
		dictionary->setTryWithoutSuffix(prop->getBool(OPTION_TRY_WITHOUT_SUFFIX, DEFAULT_TRY_WITHOUT_SUFFIX));
		dictionary->setAdvancedSearchWholeWord(prop->getBool(OPTION_ADV_SRCH_WHOLE_WORD, DEFAULT_ADV_SRCH_WHOLE_WORD));
		dictionary->setAdvancedSearchExactPhrase(prop->getBool(OPTION_ADV_SRCH_EXACT_PHRASE, DEFAULT_ADV_SRCH_EXACT_PHRASE));
		dictionary->setAdvancedSearchHighlight(prop->getBool(OPTION_ADV_SRCH_HIGHLIGHT, DEFAULT_ADV_SRCH_HIGHLIGHT));
		dictionary->setBoldDecoration(true);
		dictionary->setHTMLOutput(true);
		dictionary->setCurrentDictionary(prop->getInt(OPTION_LAST_USED_DICTIONARY, dictionary->getCurrentDictionary()));
	}

	// Create history object
	history = new History(wordsInHistory, MAX_WORD_LEN);

	// Set system style if not KDE
	systemStyleName = W2U(prop->getString(OPTION_SYSTEM_STYLE_NAME, DEFAULT_SYSTEM_STYLE_NAME));

	// Set font, read from file settings and set it
	font = QFont(
		W2U(prop->getString(OPTION_FONT_NAME, DEFAULT_FONT_NAME)),
		prop->getInt(OPTION_FONT_SIZE, DEFAULT_FONT_SIZE),
		prop->getInt(OPTION_FONT_WEIGHT, DEFAULT_FONT_WEIGHT),
		prop->getBool(OPTION_FONT_ITALIC, DEFAULT_FONT_ITALIC));

	// Set caption of window
	setCaption(tr(CAPTION));
	// Set icon of window
	QPixmap p1(kbgoffice_icon_mini);
	setIcon(p1);

	// Set positions and size of window
	int X = 0, Y = 0, W = 0, H = 0;
	sscanf(DEFAULT_GEOMETRY, "%d, %d, %d, %d", &X, &Y, &W, &H);
	sscanf(prop->getString(OPTION_GEOMETRY, DEFAULT_GEOMETRY), "%d, %d, %d, %d", &X, &Y, &W, &H);
	resize(QSize(W, H));
	move(QPoint(X, Y));

	// Create and set menu
	QPopupMenu *file = new QPopupMenu(this);
	CHECK_PTR(file);
	file->insertItem(tr("&Info"), this, SLOT(menu_Info()), CTRL + Key_I);
	file->insertSeparator();
	file->insertItem(tr("&Quit"), qApp, SLOT(quit()), CTRL + Key_Q);

	QPopupMenu *edit = new QPopupMenu(this);
	CHECK_PTR(edit);
	edit->insertItem(tr("&Copy"), this, SLOT(menu_Copy()), CTRL + Key_C);
	edit->insertItem(tr("&Paste"), this, SLOT(menu_Paste()), CTRL + Key_V);
	edit->insertSeparator();
	edit->insertItem(tr("Select &All"), this, SLOT(menu_SelectAll()), CTRL + Key_A);
	edit->insertItem(tr("C&lear"), this, SLOT(menu_Clear()), CTRL + Key_D);
	edit->insertSeparator();
	edit->insertItem(tr("Advanced &Search"), this, SLOT(button_Search()), CTRL + Key_S);
	edit->insertSeparator();
	edit->insertItem(tr("Search in Dic&tionary"), this, SLOT(button_SearchDictionary()), CTRL + Key_T);
	edit->insertItem(tr("Search in &Encyclopedia"), this, SLOT(button_SearchEncyclopedia()), CTRL + Key_E);
	edit->insertItem(tr("Search in A&ll"), this, SLOT(button_SearchEncyclopedia()), CTRL + Key_L);
	edit->insertSeparator();
	edit->insertItem(tr("History &Back"), this, SLOT(button_Prev()), CTRL + Key_Left);
	edit->insertItem(tr("History &Forward"), this, SLOT(button_Next()), CTRL + Key_Right);

	options = new QPopupMenu(this);
	CHECK_PTR(options);
	options->setCheckable(true);
	options->insertItem(tr("&Show ToolBar"), this, SLOT(menu_ShowToolBar()));
	options->insertSeparator();
	options->insertItem(tr("Select &Font..."), this, SLOT(menu_Font()));
	options->insertItem(tr("&Configure..."), this, SLOT(menu_Options()));

	translations = new QPopupMenu(this);
	CHECK_PTR(translations);
	options->setCheckable(true);
	translations->insertItem(tr("English to French"), this, SLOT(menu_TranslationsE2F()));
	translations->insertItem(tr("English to German"), this, SLOT(menu_TranslationsE2G()));
	translations->insertItem(tr("English to Italian"), this, SLOT(menu_TranslationsE2I()));
	translations->insertItem(tr("English to Portuguese"), this, SLOT(menu_TranslationsE2P()));
	translations->insertItem(tr("English to Spanish"), this, SLOT(menu_TranslationsE2S()));
	translations->insertSeparator();
	translations->insertItem(tr("French to English"), this, SLOT(menu_TranslationsF2E()));
	translations->insertItem(tr("French to German"), this, SLOT(menu_TranslationsF2G()));
	translations->insertItem(tr("French to Italian"), this, SLOT(menu_TranslationsF2I()));
	translations->insertItem(tr("French to Portuguese"), this, SLOT(menu_TranslationsF2P()));
	translations->insertItem(tr("French to Spanish"), this, SLOT(menu_TranslationsF2S()));
	translations->insertSeparator();
	translations->insertItem(tr("German to English"), this, SLOT(menu_TranslationsG2E()));
	translations->insertItem(tr("German to French"), this, SLOT(menu_TranslationsG2F()));
	translations->insertSeparator();
	translations->insertItem(tr("Italian to English"), this, SLOT(menu_TranslationsI2E()));
	translations->insertItem(tr("Italian to French"), this, SLOT(menu_TranslationsI2F()));
	translations->insertSeparator();
	translations->insertItem(tr("Portuguese to English"), this, SLOT(menu_TranslationsP2E()));
	translations->insertSeparator();
	translations->insertItem(tr("Spanish to English"), this, SLOT(menu_TranslationsS2E()));
	translations->insertItem(tr("Spanish to French"), this, SLOT(menu_TranslationsS2F()));

	QPopupMenu *help = new QPopupMenu(this);
	CHECK_PTR(help);
	help->insertItem(tr("What's &This?"), this, SLOT(whatsThis()));
	help->insertItem(tr("A&bbreviations"), this, SLOT(menu_Abbreviations()));
	help->insertSeparator();
	help->insertItem(tr("&About"), this, SLOT(menu_About()));

#ifdef WITH_KDE
	KMenuBar *menu = new KMenuBar(this);
#else
	QMenuBar *menu = new QMenuBar(this);
#endif

	toolBarOfDictionaries = new QToolBar(this, tr("Dictionaries"));
	CHECK_PTR(toolBarOfDictionaries);
	toolBarOfDictionaries->setFrameShape(QButtonGroup::NoFrame);

	QWidget *central = new QWidget(this);
	CHECK_PTR(central);
	setCentralWidget(central);

	buttons = new QButtonGroup(1, QButtonGroup::Vertical, toolBarOfDictionaries);
	CHECK_PTR(buttons);
	buttons->setExclusive(true);
	buttons->setInsideSpacing(0);
	buttons->setInsideMargin(0);
	buttons->setFrameShape(QButtonGroup::NoFrame);
	connect(buttons, SIGNAL(clicked(int)), this, SLOT(menu_Dict(int)));

	panelButtons = new QButtonGroup(1, QButtonGroup::Vertical, central);
	CHECK_PTR(panelButtons);
	panelButtons->setExclusive(true);
	panelButtons->setInsideSpacing(0);
	panelButtons->setInsideMargin(0);
	panelButtons->setFrameShape(QButtonGroup::NoFrame);
	connect(panelButtons, SIGNAL(clicked(int)), this, SLOT(menu_Dict(int)));

	listOfDictionaries = new QPopupMenu(this);
	CHECK_PTR(listOfDictionaries);
	listOfDictionaries->setCheckable(true);
	if (dataOk) {
		for (int i = 0; i < dictionary->sizeOfDictionaries(); i++) {
			Dictionary d = dictionary->getDictionary(i);
			int id = listOfDictionaries->insertItem(QString("&%1. " + W2U(d.name.c_str())).arg(i + 1), this, SLOT(menu_Dict(int)));
			listOfDictionaries->setItemParameter(id, i);
			listOfDictionaries->setItemChecked(id, dictionary->getCurrentDictionary() == i);

			QToolButton *toolb = new QToolButton(buttons);
			toolb->setPixmap(QPixmap(d.icon.c_str()));
			toolb->setTextLabel(W2U(d.name.c_str()));
			toolb->setToggleButton(true);
			toolb->setOn(dictionary->getCurrentDictionary() == i);
			buttons->insert(toolb, i);

			QToolButton *panelb = new QToolButton(panelButtons);
			panelb->setPixmap(QPixmap(d.icon.c_str()));
			panelb->setTextLabel(W2U(d.name.c_str()));
			panelb->setToggleButton(true);
			panelb->setOn(dictionary->getCurrentDictionary() == i);
			panelButtons->insert(panelb, i);

		}
	}
	listOfDictionaries->insertSeparator();
	listOfDictionaries->insertItem(tr("&Test..."), this, SLOT(menu_Test()));

	if (showToolBar) {
		if(dockToolBar) {
			panelButtons->show();
		} else {
			toolBarOfDictionaries->show();
		}
	} else {
		panelButtons->hide();
		toolBarOfDictionaries->hide();
	}
	options->setItemChecked(options->idAt(0), showToolBar);

	menu->insertItem(tr("&File"), file);
	menu->insertItem(tr("&Edit"), edit);
	menu->insertItem(tr("&Dictionaries"), listOfDictionaries);
	if (advancedOptions) {
		menu->insertItem(tr("&Translation"), translations);
	}
	menu->insertItem(tr("&Settings"), options);
	menu->insertItem(tr("&Help"), help);

	// Create and set Word (text field)
	word = new CustomLineEdit(W2U(""), central);
	CHECK_PTR(word);
	word->setMaxLength(MAX_WORD_LEN);
	connect(word, SIGNAL(returnPressed()), this, SLOT(word_EnterPressed()));
	connect(word, SIGNAL(textChanged(const QString&)), this, SLOT(word_TextChanged(const QString&)));
	connect(word, SIGNAL(vk_UpPressed()), this, SLOT(word_UpPressed()));
	connect(word, SIGNAL(vk_DownPressed()), this, SLOT(word_DownPressed()));
	QWhatsThis::add(word, tr("Enter word that you want to translate here."));

	// Create and set button clear
	bclear = new QPushButton(tr("X"), central);
	CHECK_PTR(bclear);
	bclear->setFocusPolicy(NoFocus);
	connect(bclear, SIGNAL(clicked()), this, SLOT(menu_Clear()));
	QWhatsThis::add(bclear, tr("Clears the input field."));

	// Create and set button advanced search
	bsearch = new QPushButton(tr("?"), central);
	CHECK_PTR(bsearch);
	bsearch->setFocusPolicy(NoFocus);
	connect(bsearch, SIGNAL(clicked()), this, SLOT(button_Search()));
	QWhatsThis::add(bsearch, tr(
	"<qt>Performs <b>Advanced Search</b>.<br>\n"
	"It is a very powerful function that searchs entered word(s) in whole dictionary.<br><br>\n"
	"<b>Advanced Search</b> accepts as control chars \"<b>+</b>\", \"<b>-</b>\" and \"<b>\"</b>\".<br>\n"
	"<b>+go +in</b> means <b>go</b> AND <b>in</b><br>\n"
	"<b>\"go in\"</b> means exact phrase <b>go in</b><br>\n"
	"<b>+go +in -out</b> means <b>go</b> AND <b>in</b> AND NO <b>out</b></qt>"));

	// Create and set button search in Dcitionary
	if (advancedOptions) {
		bsearchDict = new QPushButton(tr("D"), central);
		CHECK_PTR(bsearchDict);
		bsearchDict->setFocusPolicy(NoFocus);
		connect(bsearchDict, SIGNAL(clicked()), this, SLOT(button_SearchDictionary()));
		QWhatsThis::add(bsearchDict, tr("<qt>Performs <b>Advanced Search in Infoplease Dictionary (www.infoplease.com)</b>.</qt>"));

		// Create and set button search in Encyclopedia
		bsearchEncycl = new QPushButton(tr("E"), central);
		CHECK_PTR(bsearchEncycl);
		bsearchEncycl->setFocusPolicy(NoFocus);
		connect(bsearchEncycl, SIGNAL(clicked()), this, SLOT(button_SearchEncyclopedia()));
		QWhatsThis::add(bsearchEncycl, tr("<qt>Performs <b>Advanced Search in Infoplease Encyclopedia (www.infoplease.com)</b>.</qt>"));

		// Create and set button search in All
		bsearchAll = new QPushButton(tr("A"), central);
		CHECK_PTR(bsearchAll);
		bsearchAll->setFocusPolicy(NoFocus);
		connect(bsearchAll, SIGNAL(clicked()), this, SLOT(button_SearchAll()));
		QWhatsThis::add(bsearchAll, tr("<qt>Performs <b>Advanced Search in all themes on Infoplease (www.infoplease.com)</b>.</qt>"));
	} else {
		bsearchDict = NULL;
	}

	// Create and set button previous
	bprev = new QPushButton(tr("<"), central);
	CHECK_PTR(bprev);
	bprev->setFocusPolicy(NoFocus);
	bprev->setEnabled(false);
	connect(bprev, SIGNAL(clicked()), this, SLOT(button_Prev()));
	QWhatsThis::add(bprev, tr("Goes one step back in history."));

	// Create and set button next
	bnext = new QPushButton(tr(">"), central);
	CHECK_PTR(bnext);
	bnext->setFocusPolicy(NoFocus);
	bnext->setEnabled(false);
	connect(bnext, SIGNAL(clicked()), this, SLOT(button_Next()));
	QWhatsThis::add(bnext, tr("Goes one step forward in history."));

	// Create and set label found
	found = new QLabel(central);
	CHECK_PTR(found);
	found->setAlignment(QLabel::AlignCenter | QLabel::AlignVCenter);
	found->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	QWhatsThis::add(found, tr(
	"<qt>Indicates result from translation.<br>\n"
	"<b><font color=red>Red</font></b> means word is not found.<br>\n"
	"<b><font color=green>Green</font></b> means exact word is found.<br>\n"
	"<b><font color=yellow>Yellow</font></b> means similar word is found. \n"
	"For example word contains space or dash, which is not taken into \n"
	"account, or starts with capital letter, \n"
	"or you entered <b>words</b>, which is plural of <b>word</b>, but \n"
	"program is very clever and suggested <b>word</b>, without -s."));

	// Create and set picture
	pic = new QLabel(central);
	CHECK_PTR(pic);
	QPixmap p2(kbgoffice_icon);
	pic->setPixmap(p2);
	pic->setFixedSize(p2.width(), p2.height());

	// Create and set list (of words)
	list = new QListBox(central);
	CHECK_PTR(list);
	list->setHScrollBarMode(QScrollView::AlwaysOff );
	connect(list, SIGNAL(highlighted(const QString&)), this, SLOT(list_Highlighted(const QString&)));
	connect(list, SIGNAL(selected(const QString&)), this, SLOT(list_Selected(const QString&)));
	QWhatsThis::add(list, tr("List of next words in dictionary."));

	// Create and set translation result
	result = new QTextBrowser(central);
	CHECK_PTR(result);
	result->setReadOnly(true);
	QWhatsThis::add(result, tr("Result of translation."));
	result->mimeSourceFactory()->setExtensionType( "", "text/html");

	QVBoxLayout *mainLayout = new QVBoxLayout(central, borderWidth);
	QHBoxLayout *upperLayout = new QHBoxLayout(borderWidth);
	QHBoxLayout *lowerLayout = new QHBoxLayout(borderWidth);

	upperLayout->addWidget(word);
	upperLayout->addWidget(bclear);
	upperLayout->addSpacing(borderWidth);
	upperLayout->addWidget(bsearch);
	upperLayout->addSpacing(borderWidth);
	if (bsearchDict != NULL) {
		upperLayout->addWidget(bsearchDict);
		upperLayout->addWidget(bsearchEncycl);
		upperLayout->addWidget(bsearchAll);
		upperLayout->addSpacing(borderWidth);
	}
	upperLayout->addWidget(bprev);
	upperLayout->addWidget(bnext);
	upperLayout->addSpacing(borderWidth);
	upperLayout->addWidget(found);
	upperLayout->addSpacing(borderWidth);
	upperLayout->addWidget(panelButtons);
	upperLayout->addStretch();
	upperLayout->addWidget(pic);

	lowerLayout->addWidget(list);
	lowerLayout->addWidget(result);

	mainLayout->addLayout(upperLayout);
	mainLayout->addLayout(lowerLayout);

	// Set size, positions and font of widgets
	setView(systemStyleName);
	word->setFocus();

#ifdef WITH_KDE
	systemTray = new CSystemTray(this, "systemtray", p1);
	CHECK_PTR(systemTray);
	systemTray->setPixmap(p1);
	systemTray->show();

	clipboard = new CWatchClipboard();
	connect(clipboard, SIGNAL(clipboardChanged(QString)), this, SLOT(slotClipboardChanged(QString)));
	if (!clipboard->becomeDCOPClient()) {
		KMessageBox::sorry(this, tr("Could not connect to the DCOP server.\nClipboard watch will be disabled."));
	} else if ((watchClipboard) && (advancedClipboard)) {
		clipboard->startWatch();
	}
#endif

	systemClipboard = QApplication::clipboard();
	connect(systemClipboard, SIGNAL(dataChanged()), this, SLOT(slotClipboardChanged()));

	httpEnc = new QHttp("www.infoplease.com");
	CHECK_PTR(httpEnc);
	connect(httpEnc, SIGNAL(done(bool)), this, SLOT(slotSearchInfoDone(bool)));
	connect(result, SIGNAL(linkClicked(const QString&)), this, SLOT(slotSearchInfoLinkClicked(const QString&)));
	filesHTable = new QIntDict<QString> ();

}


Win::~Win() {
	if (dataOk) {
		prop->setInt(OPTION_LAST_USED_DICTIONARY, dictionary->getCurrentDictionary());
		prop->setBool(OPTION_SHOW_TOOL_BAR, showToolBar);
		prop->setBool(OPTION_WATCH_CLIPBOARD, watchClipboard);
		if (!isMinimized()) {
			char val[64];
			sprintf(val, "%d, %d, %d, %d", pos().x(), pos().y(), size().width(), size().height());
			prop->setString(OPTION_GEOMETRY, val);
		}

		delete dictionary;
		delete history;
		delete word;
		delete bclear;
		delete bsearch;
		if (bsearchDict != NULL) {
			delete bsearchDict;
			delete bsearchEncycl;
			delete bsearchAll;
		}
		delete bprev;
		delete bnext;
		delete found;
		delete pic;
		delete list;
		delete result;
		delete listOfDictionaries;
		delete options;
		delete translations;
//		delete toolBarOfDictionaries;
		delete buttons;
		delete panelButtons;
#ifdef WITH_KDE
		if (systemTray != NULL) {
			delete systemTray;
		}
		if (clipboard != NULL) {
			delete clipboard;
		}
#endif

		if (httpEnc) {
			delete httpEnc;
		}
		for(int i = (int)filesHTable->count(); i >= 0; i--) {
			QString *qstrFile = filesHTable->find(i);
			filesHTable->remove(i);
			if (qstrFile != NULL) {
				unlink(*qstrFile);
				delete qstrFile;
			}
		}
		if (filesHTable) {
			delete filesHTable;
		}
	}
}


bool Win::isDataOk() {
	return dataOk;
}


void Win::menu_ShowToolBar() {
	showToolBar = (!showToolBar);
	if (showToolBar) {
		if(dockToolBar) {
			panelButtons->show();
		} else {
			toolBarOfDictionaries->show();
		}
	} else {
		if(dockToolBar) {
			panelButtons->hide();
		} else {
			toolBarOfDictionaries->hide();
		}
	}
	options->setItemChecked(options->idAt(0), showToolBar);
}


void Win::menu_Dict(int id) {
	dictionary->setCurrentDictionary(id);
	for (int i = 0; i < dictionary->sizeOfDictionaries(); i++) {
		listOfDictionaries->setItemChecked(listOfDictionaries->idAt(i), dictionary->getCurrentDictionary() == i);
		QToolButton *b = (QToolButton*)buttons->find(i);
		b->setOn(dictionary->getCurrentDictionary() == i);
		b = (QToolButton*)panelButtons->find(i);
		b->setOn(dictionary->getCurrentDictionary() == i);
	}
	word_EnterPressed();
}


void Win::button_Prev() {
	if (history->isNew()) {
		history->prev();
		if (strcmp(history->getCurrentItem(), U2W(word->text())) == 0) {
			history->prev();
		}
	} else {
		if ((history->isFirst()) && (!bnext->isEnabled())) {
			if (strcmp(history->getCurrentItem(), U2W(word->text())) == 0) {
				history->prev();
			}
		} else {
			history->prev();
		}
	}
	if ((!bnext->isEnabled()) && (!history->isFirst())) {
		bnext->setEnabled(true);
	}
	if ((history->isLast()) && (history->getCurrentSize() > 1)) {
		bprev->setEnabled(false);
	}
	fromHistoryBuffer = true;
	word->setFocus();
	word->setText(W2U(history->getCurrentItem()));
	word->selectAll();
}


void Win::button_Next() {
	if (!bprev->isEnabled()) {
		bprev->setEnabled(true);
	}
	history->next();
	if (history->isFirst()) {
		bnext->setEnabled(false);
	}
	fromHistoryBuffer = true;
	word->setFocus();
	word->setText(W2U(history->getCurrentItem()));
	word->selectAll();
}

void Win::word_TextChanged(const QString &newText) {
	if (fromListHighlighted) {
		fromListHighlighted = false;
		return;
	}
	if ((newText[0] == '+') || (newText[0] == '"') || (newText[0] == '-')) {
		if ((!fromListSelected) && (!fromClipboard) && (!fromHistoryBuffer) && (advSrchIgnoreKey)) {
			return;
		}
		if ((fromHistoryBuffer) && (advSrchIgnoreHistory)) {
			fromHistoryBuffer = false;
			return;
		}
	}
	if ((translateOnEveryKey) || (fromListSelected) || (fromHistoryBuffer) || (fromClipboard)) {
		fromTextChanged = true;
		word_EnterPressed();
	}
}


void Win::word_EnterPressed() {
	char *Translation = NULL;
	if ((!fromTextChanged) && (advSrchEnterSearch) && ((word->text()[0] == '+') || (word->text()[0] == '"') || (word->text()[0] == '-'))) {
		button_Search();
		return;
	}
	if (dictionary->getAdvancedSearchState()) {
		dictionary->setAdvancedSearchState(false);
	}
	if (dictionary->findWord(U2W(word->text()), &Translation)) {
		if (strcmp(U2W(word->text()), dictionary->getWord()) == 0) {
			found->setBackgroundColor(COLOR_FOUND);
			found->setText(tr(FOUND));
		} else {
			found->setBackgroundColor(COLOR_SIMILAR);
			found->setText(tr(SIMILAR));
		}
	} else {
		found->setBackgroundColor(COLOR_NOT_FOUND);
		found->setText(tr(NOT_FOUND));
	}
	if (Translation[0] != '\0') {
		if ((strcmp(Translation, U2W(result->text())) != 0) || (fromListSelected) || (fromSystem)) {
			result->setText(W2U(Translation));
			result->sync();
			result->setCursorPosition(0, 0);
			if (list->isEnabled()) {
				int i = 0;
				list->clear();
				do {
					i++;
					list->insertItem(W2U(dictionary->getWord()));
				} while ((dictionary->goToNextWord()) && (i < wordsInList));
				fromSystemHighlighted = true;
				fromAdvancedSearch = false;
				list->setTopItem(0);
				list->setCurrentItem(0);
			}
		}
		if ((fromListSelected) || (fromHistoryBuffer) || (fromClipboard) || (fromSystem) || (!fromTextChanged)) {
			word->selectAll();
			fromListSelected = false;
			fromHistoryBuffer = false;
			if (fromSystem) {
				fromSystem = false;
				fromTextChanged = true;
			}
		}
		if ((!fromTextChanged) || (fromClipboard)) {
			addWordToHistory(U2W(word->text()));
			fromClipboard = false;
		}
	} else {
		fromSystem = false;
	}
	fromTextChanged = false;
}


void Win::word_UpPressed() {
	uint uiCount = list->count();
	if (uiCount <= 0) {
		return;
	}
	int iCurrentItem = list->currentItem();
	if (iCurrentItem > 0) {
		list->setCurrentItem(iCurrentItem - 1);
	} else if (iCurrentItem == -1) {
		list->setCurrentItem(0);
	}
}


void Win::word_DownPressed() {
	uint uiCount = list->count();
	if (uiCount <= 0) {
		return;
	}
	int iCurrentItem = list->currentItem();
	if (iCurrentItem < (int)(uiCount - 1)) {
		list->setCurrentItem(iCurrentItem + 1);
	} else if (iCurrentItem == -1) {
		list->setCurrentItem((int)(uiCount - 1));
	}
}


void Win::button_Search() {
	word->setFocus();
	word->selectAll();
	if (dictionary->setAdvancedSearchText(U2W(word->text()))) {
		addWordToHistory(U2W(word->text()));
		int i = 0;
		list->clear();
		while ((dictionary->searchNextWord()) && (i < wordsInList)) {
			i++;
			list->insertItem(W2U(dictionary->getWord()));
		}
		if (i > 0) {
			fromSystemHighlighted = false;
			fromAdvancedSearch = true;
			list->setTopItem(0);
			list->setCurrentItem(0);
		} else {
			result->clear();
		}
		found->setBackgroundColor(i == 0 ? COLOR_NOT_FOUND : COLOR_FOUND);
		found->setText(QString("%1").arg(i));
	}
}


void Win::list_Selected(const QString &newText) {
	fromListSelected = true;
	if (word->text() != newText) {
		word->setText(newText);
	} else {
		word_EnterPressed();
	}
}


void Win::list_Highlighted(const QString &newText) {
	char *Translation = NULL;
	if (fromSystemHighlighted) {
		fromSystemHighlighted = false;
	} else {
		dictionary->findWord(U2W(newText), &Translation);
		result->setText(W2U(Translation));
		result->sync();
		result->setCursorPosition(0, 0);
		if (fromAdvancedSearch) {
			fromAdvancedSearch = false;
		} else if ((!dictionary->getAdvancedSearchState()) || (!advSrchDoNotReplacePhrase)) {
			fromListHighlighted = true;
			word->setText(newText);
			word->selectAll();
		}
	}
}


void Win::addWordToHistory(const char *word) {
	if (wordsInHistory > 0) {
		history->addItem(word);
		bnext->setEnabled(false);
		bprev->setEnabled(true);
	}
}


void Win::menu_Copy() {
	if (result->hasSelectedText()) {
		result->copy();
	} else if (word->hasMarkedText()) {
		word->copy();
	}
}


void Win::menu_Paste() {
	fromClipboard = true;
	word->setFocus();
	word->clear();
	word->paste();
}


void Win::menu_Clear() {
	word->setFocus();
	word->clear();
}


void Win::menu_SelectAll() {
	if (word->hasFocus()) {
		word->selectAll();
	} else if (result->hasFocus()) {
		word->deselect();
		result->selectAll();
	}
}


void Win::menu_Options() {

	QTabDialog options(this, "options", true);
	options.setCaption(tr("Settings"));

	options.setOkButton(tr("&OK"));
	options.setCancelButton(tr("&Cancel"));

	QWidget general(&options);
	QVBoxLayout generalLayout(&general, SPACE_IN_DIALOGS);

	QCheckBox toek(tr("Translate on every keystroke"), &general);
	toek.setChecked(translateOnEveryKey);
	QWhatsThis::add(&toek, tr("Translate every time you press button."));

	QCheckBox sm(tr("Separate different meanings"), &general);
	sm.setChecked(dictionary->getSeparateMeanings());
	QWhatsThis::add(&sm, tr("Separate different meanings of the result with additional line."));

	QCheckBox dttmp(tr("Dock toolbar to main panel"), &general);
	dttmp.setChecked(dockToolBar);
	QWhatsThis::add(&dttmp, tr("Dock toolbar with dictionaries to main panel, next to indicator."));

	act = new QCheckBox (tr("Automatic clipboard translation"), &general);
	act->setChecked(watchClipboard);
	QWhatsThis::add(act, tr("Watch for changes in system clipboard and automaticaly translate selected text."));

#ifdef WITH_KDE
	aact = new QCheckBox(tr("Use advanced clipboard watch"), &general);
	CHECK_PTR(aact);
	aact->setChecked(advancedClipboard);
	QWhatsThis::add(aact, tr(
	"<qt>This option works only when above checkbox is checked.<br>\n"
	"When you select something dictionary will try to translate it, \n"
	"so you don't need to press Ctrl+C.<br>\n"
	"<font color=\"red\">WARNING!!!</font> This opition is experimental \n"
	"and program may crash. Use with care.</qt>"));

	connect(act, SIGNAL(toggled (bool)), this, SLOT(slotWatchClipboardOptionChanged(bool)));

	if (!act->isOn()) {
		aact->setEnabled(FALSE);
	}
#endif

	QCheckBox uss(tr("Show splash screen on start"), &general);
	uss.setChecked(useSplashScreen);
	QWhatsThis::add(&uss, tr("Choose starting with or without splash screen."));

	QHBoxLayout subLayout(SPACE_IN_DIALOGS);
	QGridLayout gridLayout(SPACE_IN_DIALOGS, 3, 6);

	QLabel lwih(tr("Words in history:"), &general);
	lwih.setAlignment(QLabel::AlignRight | QLabel::AlignVCenter);

	QSpinBox wih(MIN_WORDS_IN_HISTORY, MAX_WORDS_IN_HISTORY, 5, &general);
	wih.setValue(wordsInHistory);
	QWhatsThis::add(&wih, tr("Number of words kept in history."));

	QLabel lwil(tr("Words in list:"), &general);
	lwil.setAlignment(QLabel::AlignRight | QLabel::AlignVCenter);

	QSpinBox wil(MIN_WORDS_IN_LIST, MAX_WORDS_IN_LIST, 5, &general);
	wil.setValue(wordsInList);
	QWhatsThis::add(&wil, tr("Number of words shown in list."));

	QLabel llpw(tr("Left panel width:"), &general);
	llpw.setAlignment(QLabel::AlignRight | QLabel::AlignVCenter);

	QSpinBox lpw(MIN_LEFT_PANEL_WIDTH, MAX_LEFT_PANEL_WIDTH, 5, &general);
	lpw.setValue(leftPanelWidth);
	QWhatsThis::add(&lpw, tr("Width of the left panel with words in pixels."));

	QLabel lifw(tr("Input field width:"), &general);
	lifw.setAlignment(QLabel::AlignRight | QLabel::AlignVCenter);

	QSpinBox ifw(MIN_INPUT_FIELD_WIDTH, MAX_INPUT_FIELD_WIDTH, 5, &general);
	ifw.setValue(inputFieldWidth);
	QWhatsThis::add(&ifw, tr("Width ot the input text box in pixels."));

	QLabel lbw(tr("Border width:"), &general);
	lbw.setAlignment(QLabel::AlignRight | QLabel::AlignVCenter);

	QSpinBox bw(MIN_BORDER_WIDTH, MAX_BORDER_WIDTH, 1, &general);
	bw.setValue(borderWidth);
	QWhatsThis::add(&bw, tr("Width of the border in pixels."));

	QLabel alpw(tr("pixels"), &general);
	alpw.setAlignment(QLabel::AlignLeft | QLabel::AlignVCenter);

	QLabel aifw(tr("pixels"), &general);
	aifw.setAlignment(QLabel::AlignLeft | QLabel::AlignVCenter);

	QLabel abw(tr("pixels"), &general);
	abw.setAlignment(QLabel::AlignLeft | QLabel::AlignVCenter);

	gridLayout.addWidget(&lwih, 0, 0);
	gridLayout.addWidget(&wih, 0, 1);
	gridLayout.addWidget(&lwil, 1, 0);
	gridLayout.addWidget(&wil, 1, 1);
	gridLayout.addWidget(&llpw, 2, 0);
	gridLayout.addWidget(&lpw, 2, 1);
	gridLayout.addWidget(&alpw, 2, 2);
	gridLayout.addWidget(&lifw, 3, 0);
	gridLayout.addWidget(&ifw, 3, 1);
	gridLayout.addWidget(&aifw, 3, 2);
	gridLayout.addWidget(&lbw, 4, 0);
	gridLayout.addWidget(&bw, 4, 1);
	gridLayout.addWidget(&abw, 4, 2);

	generalLayout.addWidget(&toek);
	generalLayout.addWidget(&sm);
	generalLayout.addWidget(&dttmp);
	generalLayout.addWidget(act);
#ifdef WITH_KDE
	generalLayout.addWidget(aact);
#endif
	generalLayout.addWidget(&uss);
	generalLayout.addSpacing(SPACE_IN_DIALOGS*2);
	subLayout.addLayout(&gridLayout);
	subLayout.addStretch();
	generalLayout.addLayout(&subLayout);

#ifndef WITH_KDE
	QHBoxLayout styleLayout(SPACE_IN_DIALOGS);

	QLabel lss(tr("Look and Feel:"), &general);
	lss.setAlignment(QLabel::AlignLeft | QLabel::AlignVCenter);

	QComboBox ss(false, &general);
	QStringList styles = QStyleFactory::keys();
	styles.sort();
	QStringList::Iterator sty;
	for (sty = styles.begin(); sty != styles.end(); ++sty) {
		ss.insertItem(*sty);
		if (*sty == systemStyleName) {
			ss.setCurrentItem(ss.count() - 1);
		}
	}
	QWhatsThis::add(&ss, tr("Look and Feel of the program."));

	styleLayout.addWidget(&lss);
	styleLayout.addWidget(&ss);
	styleLayout.addStretch();
	generalLayout.addSpacing(SPACE_IN_DIALOGS*2);
	generalLayout.addLayout(&styleLayout);
#endif

	generalLayout.addStretch();

	options.addTab(&general, tr("General"));


	QWidget advancedSearch(&options);
	QVBoxLayout advancedSearchLayout(&advancedSearch, SPACE_IN_DIALOGS);

	QCheckBox sww(tr("Search for whole word(s)"), &advancedSearch);
	sww.setChecked(dictionary->getAdvancedSearchWholeWord());
	QWhatsThis::add(&sww, tr(
	"<qt>When opition is checked word <b>go</b> means exactly go.<br>\n"
	"When opition is unchecked <b>go</b> will be found as part of \n"
	"other words. For example: <b>go</b>al, car<b>go</b>, etc.</qt>"));

	QCheckBox sep(tr("Search for exact phrase"), &advancedSearch);
	sep.setChecked(dictionary->getAdvancedSearchExactPhrase());
	QWhatsThis::add(&sep, tr(
	"<qt>When opition is checked, entered phrase <b>go in</b> means <b>\"go in\"</b>.<br>\n"
	"When opition is unchecked it means <b>+go +in</b>.<br><br>\n"
	"This behaviour is valid only if you omit control chars.</qt>"));

	QCheckBox chr(tr("Color highlight result"), &advancedSearch);
	chr.setChecked(dictionary->getAdvancedSearchHighlight());
	QWhatsThis::add(&chr, tr("Highlight result of the advanced search in red."));

	QLabel lwasd(tr("When advanced search syntax detected:"), &advancedSearch);

	QCheckBox itoek(tr("Ignore translate on every keystroke"), &advancedSearch);
	itoek.setChecked(advSrchIgnoreKey);
	QWhatsThis::add(&itoek, tr(
	"Ignore translate on every keystroke if one of control chars is detected."));

	QCheckBox iht(tr("Ignore automatic translation from history"), &advancedSearch);
	iht.setChecked(advSrchIgnoreHistory);
	QWhatsThis::add(&iht, tr(
	"Ignore automatic translation from history if one of control chars is detected."));

	QCheckBox dnrp(tr("Do not replace phrase in input field"), &advancedSearch);
	dnrp.setChecked(advSrchDoNotReplacePhrase);
	QWhatsThis::add(&dnrp, tr(
	"Do not replace phrase in input field if it contains one of control chars, "
	"when you navigate in list."));

	QCheckBox ewas(tr("Enter performs advanced search"), &advancedSearch);
	ewas.setChecked(advSrchEnterSearch);
	QWhatsThis::add(&ewas, tr(
	"Enter performs advanced search if one of control chars is detected. "
	"So simply enter <b>+</b> before word and press Enter."));

	advancedSearchLayout.addWidget(&sww);
	advancedSearchLayout.addWidget(&sep);
	advancedSearchLayout.addWidget(&chr);
	advancedSearchLayout.addSpacing(SPACE_IN_DIALOGS*2);
	advancedSearchLayout.addWidget(&lwasd);
	advancedSearchLayout.addWidget(&itoek);
	advancedSearchLayout.addWidget(&iht);
	advancedSearchLayout.addWidget(&dnrp);
	advancedSearchLayout.addWidget(&ewas);
	advancedSearchLayout.addStretch();

	options.addTab(&advancedSearch, tr("Advanced Search"));


	QWidget languages(&options);
	QVBoxLayout languagesLayout(&languages, SPACE_IN_DIALOGS);

	QLabel english(tr("English:"), &languages);

	QCheckBox tws(tr("If not found, try without suffix (-s, -ing, -ed)"), &languages);
	tws.setChecked(dictionary->getTryWithoutSuffix());
	QWhatsThis::add(&tws, tr("If original word is not found, program will try to remove suffix and will search again."));

	QLabel transl(tr("Translation:"), &languages);

	QCheckBox uao(tr("Enable translation (requires restart)"), &languages);
	uao.setChecked(advancedOptions);
	QWhatsThis::add(&uao, tr(
	"Enable search in Infoplease.com and translation. "
	"In order to work this option requires restart of the application."));

	QCheckBox ueb(tr("Use external browser"), &languages);
	ueb.setChecked(useExternalBrowser);
	QWhatsThis::add(&ueb, tr("Use internal text browser to display online translations."));

	QLabel lbn(tr("External browser:"), &languages);

	QLineEdit bn(browserName, &languages);
	bn.setMaxLength(PATH_MAX);
	QWhatsThis::add(&bn, tr("Enter the name and probably path to your famous browser."));

	languagesLayout.addWidget(&english);
	languagesLayout.addWidget(&tws);
	languagesLayout.addSpacing(SPACE_IN_DIALOGS*2);
	languagesLayout.addWidget(&transl);
	languagesLayout.addWidget(&uao);
	languagesLayout.addSpacing(SPACE_IN_DIALOGS*2);
	languagesLayout.addWidget(&ueb);
	languagesLayout.addWidget(&lbn);
	languagesLayout.addWidget(&bn);
	languagesLayout.addStretch();

	options.addTab(&languages, tr("Languages"));


	// Execute dialog and check for OK button
	if (options.exec() == QDialog::Accepted) {
		// Save and apply new values
		if (translateOnEveryKey != toek.isChecked()) {
			translateOnEveryKey = toek.isChecked();
			prop->setBool(OPTION_TRANSLATE_ON_EVERY_KEY, translateOnEveryKey);
		}
		if (dictionary->getSeparateMeanings() != sm.isChecked()) {
			dictionary->setSeparateMeanings(sm.isChecked());
			prop->setBool(OPTION_SEPARATE_MEANINGS, sm.isChecked());
		}
		if (dockToolBar != dttmp.isChecked()) {
			dockToolBar = dttmp.isChecked();
			prop->setBool(OPTION_DOCK_TOOL_BAR, dockToolBar);
		}
#ifdef WITH_KDE
		if (advancedClipboard != aact->isChecked()) {
			advancedClipboard = aact->isChecked();
			prop->setBool(OPTION_ADVANCED_CLIPBOARD, advancedClipboard);
		}
#endif
		if (watchClipboard != act->isChecked()) {
			watchClipboard = act->isChecked();
		}

#ifdef WITH_KDE
		setWatchClipboard(watchClipboard, advancedClipboard);
#endif

		if (useSplashScreen != uss.isChecked()) {
			useSplashScreen = uss.isChecked();
			prop->setBool(OPTION_USE_SPLASH_SCREEN, useSplashScreen);
		}
		if (inputFieldWidth != ifw.value()) {
			inputFieldWidth = ifw.value();
			prop->setInt(OPTION_INPUT_FIELD_WIDTH, inputFieldWidth);
		}
		if (borderWidth != bw.value()) {
			borderWidth = bw.value();
			prop->setInt(OPTION_BORDER_WIDTH, borderWidth);
		}
		if (leftPanelWidth != lpw.value()) {
			leftPanelWidth = lpw.value();
			prop->setInt(OPTION_LEFT_PANEL_WIDTH, leftPanelWidth);
		}
		if (wordsInList != wil.value()) {
			wordsInList = wil.value();
			prop->setInt(OPTION_WORDS_IN_LIST, wordsInList);
		}
		if (wordsInHistory != wih.value()) {
			wordsInHistory = wih.value();
			prop->setInt(OPTION_WORDS_IN_HISTORY, wordsInHistory);
			history->setMaxSize(wordsInHistory);
			if (wordsInHistory == 0) {
				bnext->setEnabled(false);
				bprev->setEnabled(false);
			}
		}
#ifndef WITH_KDE
		if (systemStyleName != ss.currentText()) {
			systemStyleName = ss.currentText();
			prop->setString(OPTION_SYSTEM_STYLE_NAME, U2W(systemStyleName));
		}
#endif

		// Advanced Search
		if (dictionary->getAdvancedSearchWholeWord() != sww.isChecked()) {
			dictionary->setAdvancedSearchWholeWord(sww.isChecked());
			prop->setBool(OPTION_ADV_SRCH_WHOLE_WORD, sww.isChecked());
		}
		if (dictionary->getAdvancedSearchExactPhrase() != sep.isChecked()) {
			dictionary->setAdvancedSearchExactPhrase(sep.isChecked());
			prop->setBool(OPTION_ADV_SRCH_EXACT_PHRASE, sep.isChecked());
		}
		if (dictionary->getAdvancedSearchHighlight() != chr.isChecked()) {
			dictionary->setAdvancedSearchHighlight(chr.isChecked());
			prop->setBool(OPTION_ADV_SRCH_HIGHLIGHT, chr.isChecked());
		}
		if (advSrchIgnoreKey != itoek.isChecked()) {
			advSrchIgnoreKey = itoek.isChecked();
			prop->setBool(OPTION_ADV_SRCH_ASD_IGNORE_KEY, advSrchIgnoreKey);
		}
		if (advSrchIgnoreHistory != iht.isChecked()) {
			advSrchIgnoreHistory = iht.isChecked();
			prop->setBool(OPTION_ADV_SRCH_ASD_IGNORE_HIS, advSrchIgnoreHistory);
		}
		if (advSrchDoNotReplacePhrase != dnrp.isChecked()) {
			advSrchDoNotReplacePhrase = dnrp.isChecked();
			prop->setBool(OPTION_ADV_SRCH_ASD_DO_NOT_REPL, advSrchDoNotReplacePhrase);
		}
		if (advSrchEnterSearch != ewas.isChecked()) {
			advSrchEnterSearch = ewas.isChecked();
			prop->setBool(OPTION_ADV_SRCH_ASD_ENTER, advSrchEnterSearch);
		}

		// Languages
		if (dictionary->getTryWithoutSuffix() != tws.isChecked()) {
			dictionary->setTryWithoutSuffix(tws.isChecked());
			prop->setBool(OPTION_TRY_WITHOUT_SUFFIX, tws.isChecked());
		}

		// Translation
		if (advancedOptions != uao.isChecked()) {
			advancedOptions = uao.isChecked();
			prop->setBool(OPTION_ADVANCED_OPTIONS, advancedOptions);
		}

		if (useExternalBrowser != ueb.isChecked()) {
			useExternalBrowser = ueb.isChecked();
			prop->setBool(OPTION_USE_EXTERNAL_BROWSER, useExternalBrowser);
		}

		if (!bn.text().isEmpty()) {
			browserName = bn.text();
			prop->setString(OPTION_DEFAULT_BROWSER, browserName);
		}

		// Update view of main window
		setView(systemStyleName);
		// Update word and translation
		fromSystem = true;
		word_EnterPressed();
	}

#ifdef WITH_KDE
	delete act;
	delete aact;
#endif
}


void Win::menu_Font() {
	bool okSelected;
	QFont f = QFont(font);
#ifdef WITH_KDE
	okSelected = (KFontDialog::getFont(f, false, this) == QDialog::Accepted);
#else
	f = QFontDialog::getFont(&okSelected, f, this);
#endif
	if (okSelected) {
		font = QFont(f);
		// Ignore some options which user may select
		font.setUnderline(false);
		font.setStrikeOut(false);
		// Save user selections
		prop->setString(OPTION_FONT_NAME, U2W(font.family()));
		prop->setInt(OPTION_FONT_SIZE, font.pointSize());
		prop->setInt(OPTION_FONT_WEIGHT, font.weight());
		prop->setBool(OPTION_FONT_ITALIC, font.italic());
		// Update view of main window
		setView("");
		// Update word and translation
		fromSystem = true;
		word_EnterPressed();
	}
}


void Win::menu_Test() {
	Test t(this, "test", dictionary, prop, &font, borderWidth);
	t.exec();
}


void Win::menu_Abbreviations() {
	char *c = NULL;
	c = dictionary->getAbbreviations();
	if (c[0] == '\0') {
		result->setText(tr(MISSING_ABRR_FILE));
	} else {
		result->setText(W2U(c));
	}
	result->sync();
	result->setCursorPosition(0, 0);
}


void Win::menu_About() {

	QDialog about(this, "about", true);
	about.setCaption(tr("About"));

	QVBoxLayout aboutLayout(&about, SPACE_IN_DIALOGS);

	QLabel picture(&about);
	QPixmap aboutPic(QString(dataDir) + "kbgoffice_about.png");
	picture.setPixmap(aboutPic);
	picture.setAlignment(QLabel::AlignTop);

	QLabel message(tr(ABOUT_MESSAGE).arg(VERSION), &about);

	QPushButton close(tr("&Close"), &about);
	close.setDefault(true);
	connect(&close, SIGNAL(clicked()), &about, SLOT(accept()));

	QHBoxLayout subLayout(SPACE_IN_DIALOGS);
	subLayout.addWidget(&picture);
	subLayout.addWidget(&message);

	QHBoxLayout buttonLayout(SPACE_IN_DIALOGS);
	buttonLayout.addStretch();
	buttonLayout.addWidget(&close);

	aboutLayout.addLayout(&subLayout);
	aboutLayout.addSpacing(SPACE_IN_DIALOGS);
	aboutLayout.addLayout(&buttonLayout);

	about.exec();
}


void Win::menu_Info() {

	QDialog info(this, "info", true);
	info.setCaption(tr("Info"));

	Dictionary d = dictionary->getDictionary(dictionary->getCurrentDictionary());
	char *p = dictionary->countWords();

	QVBoxLayout infoLayout(&info, SPACE_IN_DIALOGS);

	QLabel dictPicture(&info);
	QPixmap dictIcon(d.icon.c_str());
	dictPicture.setPixmap(dictIcon);
	dictPicture.setAlignment(QLabel::AlignTop);

	QLabel message(tr(
		"Info about current dictionary\n\n"
		"Name: %1\n"
		"Version: %2\n"
		"Words: %3").arg(W2U(d.name.c_str())).arg(d.version.c_str()).arg(p), &info);

	QPushButton close(tr("&Close"), &info);
	close.setDefault(true);
	connect(&close, SIGNAL(clicked()), &info, SLOT(accept()));

	QHBoxLayout subLayout(SPACE_IN_DIALOGS);
	subLayout.addWidget(&dictPicture);
	subLayout.addWidget(&message);

	QHBoxLayout buttonLayout(SPACE_IN_DIALOGS);
	buttonLayout.addStretch();
	buttonLayout.addWidget(&close);

	infoLayout.addLayout(&subLayout);
	infoLayout.addSpacing(SPACE_IN_DIALOGS);
	infoLayout.addLayout(&buttonLayout);

	info.exec();
}


void Win::setView(QString style) {

#ifndef WITH_KDE
	// Set Style View
	qApp->setStyle(style);
#endif

	// Set font for screen element
	word->setFont(font);
	bclear->setFont(font);
	bsearch->setFont(font);
	if (bsearchDict != NULL) {
		bsearchDict->setFont(font);
		bsearchEncycl->setFont(font);
		bsearchAll->setFont(font);
	}
	bprev->setFont(font);
	bnext->setFont(font);
	found->setFont(font);
	list->setFont(font);
	result->setFont(font);

	// Update Word (text field)
	word->setMaximumWidth(inputFieldWidth);
	word->setMinimumWidth(inputFieldWidth);

	// Update Found label
	found->setMaximumWidth(word->height());
	found->setMinimumWidth(word->height());
	found->setMaximumHeight(word->height());
	found->setMinimumHeight(word->height());

	// Update buttons
	bclear->setMaximumWidth(bclear->height());
	bclear->setMinimumWidth(bclear->height());
	bsearch->setMaximumWidth(bclear->height());
	bsearch->setMinimumWidth(bclear->height());
	if (bsearchDict != NULL) {
		bsearchDict->setMaximumWidth(bclear->height());
		bsearchDict->setMinimumWidth(bclear->height());
		bsearchEncycl->setMaximumWidth(bclear->height());
		bsearchEncycl->setMinimumWidth(bclear->height());
		bsearchAll->setMaximumWidth(bclear->height());
		bsearchAll->setMinimumWidth(bclear->height());
	}
	bprev->setMaximumWidth(bclear->height());
	bprev->setMinimumWidth(bclear->height());
	bnext->setMaximumWidth(bclear->height());
	bnext->setMinimumWidth(bclear->height());

	// List of dictionaries
	if (showToolBar) {
		if(dockToolBar) {
			panelButtons->show();
			toolBarOfDictionaries->hide();
		} else {
			toolBarOfDictionaries->show();
			panelButtons->hide();
		}
	}

	// Update list (of words)
	list->setMaximumWidth(leftPanelWidth);
	list->setMinimumWidth(leftPanelWidth);
	if ((leftPanelWidth > MIN_LEFT_PANEL_WIDTH) && (wordsInList > MIN_WORDS_IN_LIST)) {
		list->setEnabled(true);
		list->show();
	} else {
		list->setEnabled(false);
		list->hide();
	}

	// Update translation result
	result->setText(result->text());
	result->sync();
	result->setCursorPosition(0, 0);

}


#ifdef WITH_KDE

void Win::closeEvent(QCloseEvent *e) {
	e->ignore();
	hide();
}


void Win::showOptions() {
	menu_Options();
}


void Win::setWatchClipboard(
	const bool watch,
	const bool bAdvWatch
	)
{
	watchClipboard		= watch;
	advancedClipboard 	= bAdvWatch;

	if (watchClipboard && advancedClipboard) {
		clipboard->startWatch();
	} else {
		clipboard->stopWatch();
	}
}


bool Win::isWatchClipboard () {
	return watchClipboard;
}

bool Win::isAdvWatchClipboard () {
	return advancedClipboard;
}


#endif


void Win::slotClipboardChanged(QString data) {
	if (data != NULL) {
		fromClipboard = true;
		word->setFocus();
		word->clear();
		word->setText(data);
		setActiveWindow();
		raise();
	}
}


void Win::slotClipboardChanged() {
	if ((watchClipboard) && (!advancedClipboard)) {
		QString data = NULL;
		data = systemClipboard->text(QClipboard::Clipboard);
		if (!data.isNull()) {
			fromClipboard = true;
			word->setFocus();
			word->clear();
			word->setText(data);
			setActiveWindow();
			raise();
		}
	}
}


void Win::slotWatchClipboardOptionChanged(bool bOn) {
#ifdef WITH_KDE
	if (!bOn) {
		aact->setChecked(FALSE);
		aact->setDisabled(TRUE);
	} else {
		aact->setDisabled(FALSE);
	}
#endif
}


void Win::button_SearchDictionary() {
	QString qstrTmp(SEARCH_DICTIONARY);
	QString qTmp = word->text();
	QUrl::encode(qTmp);
	qstrTmp += qTmp;
	QUrl qUrl(qstrTmp);
	if (qUrl.hasHost()) {
		httpEnc->setHost(qUrl.host());
	}
	httpEnc->get( qstrTmp );
	result->setText(tr("Searching, please wait..."));
}


void Win::button_SearchEncyclopedia() {
	QString qstrTmp(SEARCH_ENCYCLOPEDIA);
	QString qTmp = word->text();
	QUrl::encode(qTmp);
	qstrTmp += qTmp;
	QUrl qUrl(qstrTmp);
	if (qUrl.hasHost()) {
		httpEnc->setHost(qUrl.host());
	}
	httpEnc->get(qstrTmp);
	result->setText(tr("Searching, please wait..."));
}


void Win::button_SearchAll() {
	QString qstrTmp(SEARCH_ALL);
	QString qTmp = word->text();
	QUrl::encode(qTmp);
	qstrTmp += qTmp;
	QUrl qUrl(qstrTmp);
	if (qUrl.hasHost()) {
		httpEnc->setHost(qUrl.host());
	}
	httpEnc->get(qstrTmp);
	result->setText(tr("Searching, please wait..."));
}


void Win::slotSearchInfoDone(bool bError) {
	if (!bError) {
		char tempFilePattern[] = "/tmp/SResXXXXXX";
		int ifd = mkstemp(tempFilePattern);
		FILE *tmpFile = NULL;
		if (ifd == -1 || (tmpFile = fdopen(ifd, "wb")) == NULL) {
#ifdef WITH_KDE
			KMessageBox::sorry(this, W2U(CREATION_TEMP_FILE_FAILED), this->caption());
#else
			QMessageBox::critical(this, this->caption(), W2U(CREATION_TEMP_FILE_FAILED));
#endif
			return;
		}

		QByteArray qbtArr = httpEnc->readAll();
		QFile file(tempFilePattern);
		file.open(IO_WriteOnly, tmpFile);
		file.writeBlock(qbtArr);
		file.close(); // this one just flushes file
		fclose(tmpFile);

		if (useExternalBrowser) {
			launchExternalBrowser(tempFilePattern);
		} else {
			result->setSource(tempFilePattern);
			unlink(tempFilePattern);
		}
	} else {
		QString qStrError = tr("Search failed. Reason:");
		qStrError += httpEnc->errorString();
#ifdef WITH_KDE
		KMessageBox::sorry(this, qStrError, this->caption());
#else
		QMessageBox::critical(this, this->caption(), qStrError);
#endif
	}
}


void Win::launchExternalBrowser(QString qstrFileName) {
	QProcess qBrowser;
	bool bSuccess = FALSE;
	qBrowser.addArgument(browserName);
	qBrowser.addArgument(qstrFileName);
	bSuccess = qBrowser.start();

	if (!bSuccess) {
#ifdef WITH_KDE
		KMessageBox::sorry(this, tr("Failed to invoke: ") + browserName, this->caption());
#else
		QMessageBox::critical(this, this->caption(), tr("Failed to invoke: ") + browserName);
#endif
		return;
	}

	int iCurSize = filesHTable->count();
	iCurSize += 1;
	filesHTable->insert(iCurSize, new QString(qstrFileName));
	result->setText(tr("Done."));
}

void Win::slotSearchInfoLinkClicked(const QString &qStrLink) {
	QString qStrTmp = qStrLink;
	QUrl qUrl(qStrTmp);

	if (!qUrl.isValid()) {
#ifdef WITH_KDE
		KMessageBox::sorry(this, tr("Invalid URL."), this->caption());
#else
		QMessageBox::critical(this, this->caption(), tr("Invalid URL."));
#endif
		return;
	}
	if (qUrl.hasHost()) {
		httpEnc->setHost(qUrl.host());
	}
	httpEnc->get(qStrTmp);
	result->setText(tr("Loading data, please wait..."));
}


void Win::menu_TranslationsE2F() {
	Translations(E2F);
}

void Win::menu_TranslationsE2G() {
	Translations(E2G);
}

void Win::menu_TranslationsE2I() {
	Translations(E2I);
}

void Win::menu_TranslationsE2P() {
	Translations(E2P);
}

void Win::menu_TranslationsE2S() {
	Translations(E2S);
}

void Win::menu_TranslationsF2E() {
	Translations(F2E);
}

void Win::menu_TranslationsF2G() {
	Translations(F2G);
}

void Win::menu_TranslationsF2I() {
	Translations(F2I);
}

void Win::menu_TranslationsF2P() {
	Translations(F2P);
}

void Win::menu_TranslationsF2S() {
	Translations(F2S);
}

void Win::menu_TranslationsG2E() {
	Translations(G2E);
}

void Win::menu_TranslationsG2F() {
	Translations(G2F);
}

void Win::menu_TranslationsI2E() {
	Translations(I2E);
}

void Win::menu_TranslationsI2F() {
	Translations(I2F);
}

void Win::menu_TranslationsP2E() {
	Translations(P2E);
}

void Win::menu_TranslationsS2E() {
	Translations(S2E);
}

void Win::menu_TranslationsS2F() {
	Translations(S2F);
}

void Win::Translations(TRANSLATIONS_TYPE tType) {

	QString qStrLink;

	switch (tType) {
		case E2F:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=en_fr&partner=demo-Lycos2-en";
			break;
		case E2G:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=en_de&partner=demo-Lycos2-en";
			break;
		case E2I:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=en_it&partner=demo-Lycos2-en";
			break;
		case E2P:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=en_pt&partner=demo-Lycos2-en";
			break;
		case E2S:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=en_es&partner=demo-Lycos2-en";
			break;
		case F2E:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=fr_en&partner=demo-Lycos2-en";
			break;
		case F2G:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=fr_de&partner=demo-Lycos2-en";
			break;
		case F2I:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=fr_it&partner=demo-Lycos2-en";
			break;
		case F2P:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=fr_pt&partner=demo-Lycos2-en";
			break;
		case F2S:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=fr_es&partner=demo-Lycos2-en";
			break;
		case G2E:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=de_en&partner=demo-Lycos2-en";
			break;
		case G2F:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=de_fr&partner=demo-Lycos2-en";
			break;
		case I2E:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=it_en&partner=demo-Lycos2-en";
			break;
		case I2F:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=it_fr&partner=demo-Lycos2-en";
			break;
		case P2E:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=pt_en&partner=demo-Lycos2-en";
			break;
		case S2E:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=es_en&partner=demo-Lycos2-en";
			break;
		case S2F:
			qStrLink = "http://translate.lycoszone.com/lycos?urltext=$$$&systran_lp=es_fr&partner=demo-Lycos2-en";
			break;
		default:
			return;
	}

	QString qTmp = word->text();
	QUrl::encode(qTmp);
	qStrLink += qTmp;
	qStrLink.replace("$$$", qTmp);
	QUrl qUrl(qStrLink);
	if (!qUrl.isValid()) {
#ifdef WITH_KDE
		KMessageBox::sorry(this, tr("Invalid URL."), this->caption());
#else
		QMessageBox::critical(this, this->caption(), tr("Invalid URL."));
#endif
		return;
	}

	if (qUrl.hasHost()) {
		httpEnc->setHost(qUrl.host());
	}
	httpEnc->get(qStrLink);
	result->setText(tr("Translating, please wait..."));
}
