/*************************************************************************
*
* BG Office Project
* Copyright (C) 2000-2004 Radostin Radnev <radnev@yahoo.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*************************************************************************/


#ifndef K_BGO_A_WIN_H
#define K_BGO_A_WIN_H

#include <qmainwindow.h>

#include "translator_manager.h"
#include "properties.h"
#include "history.h"
#include "customlineedit.h"

#ifdef WITH_KDE
#include "watchclipboard.h"
#include "systemtray.h"
#endif

#include <qhttp.h>
#include <qintdict.h>

/*
* $Id: win.h,v 1.8 2005/03/12 09:14:46 radnev Exp $
*/


class QFont;
class QLineEdit;
class QPushButton;
class QLabel;
class QListBox;
class QTextBrowser;
class QClipboard;
class QCheckBox;
class QSpinBox;
class QComboBox;
class QPopupMenu;
class QToolBar;
class QButtonGroup;
#ifdef WITH_KDE
class CSystemTray;
#endif


enum TRANSLATIONS_TYPE {
	E2F, E2G, E2I, E2P, E2S,
	F2E, F2G, F2I, F2P, F2S,
	G2E, G2F,
	I2E, I2F,
	P2E,
	S2E, S2F
};


class Win: public QMainWindow {

Q_OBJECT

public:
	Win(Properties *pProp, char *pDataDir, QWidget *parent = 0, const char *name = 0);
	~Win();
	bool isDataOk();
#ifdef WITH_KDE
	void showOptions();
	void setWatchClipboard(const bool watch, const bool bAdvWatch);
	bool isWatchClipboard();
	bool isAdvWatchClipboard();
#endif

protected:
	void setView(QString style);
	void addWordToHistory(const char *word);
#ifdef WITH_KDE
	void closeEvent(QCloseEvent *e);
#endif
	void Translations(TRANSLATIONS_TYPE tType);
	void launchExternalBrowser(QString qstrFileName);

protected slots:
	void word_TextChanged(const QString &newText);
	void word_EnterPressed();
	void word_UpPressed();
	void word_DownPressed();
	void list_Highlighted(const QString &newText);
	void list_Selected(const QString &newText);
	void menu_Copy();
	void menu_Paste();
	void menu_Clear();
	void menu_SelectAll();
	void menu_Dict(int id);
	void menu_ShowToolBar();
	void menu_Font();
	void menu_Options();
	void menu_Test();
	void menu_About();
	void menu_Abbreviations();
	void menu_Info();
	void button_Prev();
	void button_Next();
	void button_Search();
	void slotClipboardChanged(QString);
	void slotClipboardChanged();
	void button_SearchDictionary();
	void button_SearchEncyclopedia();
	void button_SearchAll();
	void slotSearchInfoDone(bool bError);
	void slotSearchInfoLinkClicked(const QString &qStrLink);
	void slotWatchClipboardOptionChanged(bool bOn);
	void menu_TranslationsE2F();
	void menu_TranslationsE2G();
	void menu_TranslationsE2I();
	void menu_TranslationsE2P();
	void menu_TranslationsE2S();
	void menu_TranslationsF2E();
	void menu_TranslationsF2G();
	void menu_TranslationsF2I();
	void menu_TranslationsF2P();
	void menu_TranslationsF2S();
	void menu_TranslationsG2E();
	void menu_TranslationsG2F();
	void menu_TranslationsI2E();
	void menu_TranslationsI2F();
	void menu_TranslationsP2E();
	void menu_TranslationsS2E();
	void menu_TranslationsS2F();

private:
	QFont font;
	CustomLineEdit *word;
	QPushButton *bclear;
	QPushButton *bsearch;
	QPushButton *bsearchDict;
	QPushButton *bsearchEncycl;
	QPushButton *bsearchAll;
	QPushButton *bprev;
	QPushButton *bnext;
	QLabel *found;
	QLabel *pic;
	QListBox *list;
	QTextBrowser *result;
	QClipboard *systemClipboard;
	QPopupMenu *listOfDictionaries;
	QPopupMenu *options;
	QToolBar *toolBarOfDictionaries;
	QButtonGroup *buttons;
	QButtonGroup *panelButtons;
#ifdef WITH_KDE
	CSystemTray *systemTray;
	CWatchClipboard *clipboard;
#endif

	QCheckBox *act;
	QCheckBox *aact;

	QHttp *httpEnc;
	QPopupMenu *translations;
	TRANSLATIONS_TYPE tType;
	bool useExternalBrowser;
	QString browserName;
	QIntDict<QString> *filesHTable;

	bool dataOk;

	Properties *prop;

	TranslatorManager *dictionary;

	History *history;

	char *dataDir;

	int inputFieldWidth;
	int leftPanelWidth;
	int wordsInList;
	int borderWidth;
	int wordsInHistory;

	QString systemStyleName;

	bool showToolBar;
	bool dockToolBar;

	bool translateOnEveryKey;
	bool watchClipboard;
	bool advancedClipboard;
	bool advancedOptions;

	bool fromSystemHighlighted;
	bool fromListHighlighted;
	bool fromHistoryBuffer;
	bool fromListSelected;
	bool fromClipboard;
	bool fromSystem;
	bool fromTextChanged;
	bool fromAdvancedSearch;

	bool advSrchIgnoreKey;
	bool advSrchIgnoreHistory;
	bool advSrchEnterSearch;
	bool advSrchDoNotReplacePhrase;

	bool useSplashScreen;
};

#endif
